package com.hamaf.hikyaku.access;

import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.utils.Dto2SqlUtils;


/**
 * <p>^CgF MultipleTableAccess is Multiple Table Access Class.</p>
 * <p>Fe[uXVpe[uANZXNXB</p>
 * <br>e[uw肵Ċe\bhĂяo܂B
 * <br>̃NXł͎IȃR~bgs܂B
 * <br>gpNXŖI<b>commit,rollBack</b>s܂B
 * <br>@\
 * <ol>
 *   <li>}@\ - insert(DetailItemsInterface)
 *   <li>XV@\ - update(DetailItemsInterface)
 *   <li>폜@\ - delete(DetailItemsInterface)
 * </ol>
 *
 * <p>Created: Tue Feb 15 16:34:14 2005</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class MultipleTableAccess extends MultipleAccessBase {

  protected String connectionParameter = null;

  private boolean _debug = false;
  private final static String CLNM = "*** MultipleTableAccess : ";

  private static final String UPD_SQL = "UPDATE {0} {1} {2}";
  private static final String IST_SQL = "INSERT INTO {0} ({1}) VALUES({2})";
  private static final String DEL_SQL = "DELETE FROM {0} {1}";

  /**
   * Describe <code>setConnectionParameter</code> method : RlNV擾p[^ݒ肷.
   * @param connPrm a <code>String</code> : RlNV擾p[^
   */
  public final void setConnectionParameter(String connPrm) {
    connectionParameter = connPrm;
  }

  /**
   * Describe <code>regist</code> method : insert TABLE.
   * <br>f[^o^B
   * <br>f[^IuWFNg̎gpɂĔrp̏
   * <code>(getInsertItemWithExclusion,getInsertValueWithExclusion)</code>𗘗pB
   * @param tblName a <code>String</code> : e[u
   * @param dii a <code>DetailItemsInterface</code>
   * @exception SQLException
   */
  public void regist(String tblName, DetailItemsInterface dii) throws SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();

    Object[] prm = {tblName, Dto2SqlUtils.getInsertItemWithExclusion(dii),
        Dto2SqlUtils.getInsertValueWithExclusion(dii)};
    String sqlStm = MessageFormat.format(IST_SQL, prm);
    debugOut(" (regist) ", sqlStm);

    try {
      stm.executeUpdate(sqlStm);
    }
    finally {
      stm.close();
    }
  }

  /**
   * Describe <code>update</code> method : update TABLE items by PrimaryKey.
   * <br>vC}L[TABLEXVB
   * <br>XVOɔr`FbNsB
   * <br>f[^IuWFNg̎gpɂĔrp̏
   * <code>(getUpdateStringWithExclusion)</code>𗘗pB
   *
   * @param tblName a <code>String</code> : e[u
   * @exception ExclusionException : rG[̏ꍇ
   * @exception SQLException
   */
  public void update(String tblName, DetailItemsInterface dii) throws ExclusionException, SQLException {
    updateExec(tblName, dii, true);
  }

  /**
   * Describe <code>updateNoExclusion</code> method : update TABLE items by PrimaryKey.
   * <br>r`FbNsȂ킸ɁAvC}L[TABLEXVB
   *
   * @param tblName a <code>String</code> : e[u
   * @param dii a <code>DetailItemsInterface</code>
   * @throws SQLException
   */
  public void updateNoExclusion(String tblName, DetailItemsInterface dii) throws SQLException {
    try {
      updateExec(tblName, dii, false);
    } catch (ExclusionException ee) { }
  }
  private void updateExec(String tableName, DetailItemsInterface dii, boolean excl) throws ExclusionException, SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();

    try {
      if (excl) {
        super.exclusion(dii, tableName);
      }

      Object[] prm = {tableName, Dto2SqlUtils.getUpdateItemsWithExclusion(dii),
          Dto2SqlUtils.getWhereString(dii, getPkeys(tableName))};
      String sqlStm = MessageFormat.format(UPD_SQL, prm);
      debugOut(" (update) ", sqlStm);

      stm.executeUpdate(sqlStm);
    } finally {
      stm.close();
    }
  }

  /**
   * Describe <code>delete</code> method : delete instance by PrimaryKey.
   * <br>vC}L[Ńf[^폜
   * <br>r`FbNs܂B
   * @param tblName a <code>String</code> : e[u
   * @param dii a <code>DetailItemsInterface</code>
   * @exception ExclusionException : rG[̏ꍇ
   * @exception SQLException if an error occurs
   */
  public void delete(String tblName, DetailItemsInterface dii) throws ExclusionException, SQLException {
    deleteExec(tblName, dii, true);
  }

  /**
   * Describe <code>deleteNoExclusion</code> method : delete TABLE items by PrimaryKey.
   * <br>r`FbNsȂ킸ɁAvC}L[TABLE폜B
   *
   * @param tblName a <code>String</code> : e[u
   * @param dii a <code>DetailItemsInterface</code>
   * @throws SQLException
   */
  public void deleteNoExclusion(String tblName, DetailItemsInterface dii) throws SQLException {
    try {
      deleteExec(tblName, dii, false);
    } catch (ExclusionException ee) { }
  }
  private void deleteExec(String tableName, DetailItemsInterface dii, boolean excl) throws ExclusionException, SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();

    try {
      if (excl) {
        super.exclusion(dii, tableName);
      }

      Object[] prm = {tableName, Dto2SqlUtils.getWhereString(dii, getPkeys(tableName))};
      String sqlStm = MessageFormat.format(DEL_SQL, prm);
      debugOut(" (delete) ", sqlStm);

      stm.executeUpdate(sqlStm);
    } finally {
      stm.close();
    }
  }

  /**
   * @param argDebug debug ݒ肵܂B
   */
  public final void setDebug() {
    _debug = true;
  }

  /**
   * Describe <code>debugOut</code> method : print sql string.
   * @param methodName
   * @param msg a <code>CharSequence</code> : message
   */
  private void debugOut(String methodName, CharSequence msg) {
    if (!_debug) {
      return;
    }
    System.out.print(CLNM);
    System.out.print(" (find) ");
    System.out.println(msg.toString());
  }
}
