package com.hamaf.hikyaku.plugIn.dto;

import java.io.File;
import java.text.MessageFormat;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import com.hamaf.hikyaku.util.XPathUtil;
import com.hamaf.hikyaku.util.XmlUtil;

/**
 * <p>^CgFInfoXml is controled Plugin Information XML.</p>
 * <p>FvOCXML̏s</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2010/01/25</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class InfoXml {

  public enum INF_TAG {displayname, exposition, author, icon,
    controlxmlpass, controlgetmode, diclass, difile};
  public enum MENU_TAG {
  	ID ("id"),
  	DISP("display"),
  	PROC("process"),
  	ACT("action"),
    TGT("target");
  	private final String val;
  	MENU_TAG(String argval) { val = argval; }
  	public String getVal() { return val; }
  }
  private final String MENU_VAL = "menus/menu[{0}]/{1}";
  private final String MENUS = "menus/menu";
  public enum INIT_TAG {
    ID ("id"),
    DISP("display"),
    PROC("process"),
    ACT("action"),
    OPT("option");
    private final String val;
    INIT_TAG(String argval) { val = argval; }
    public String getVal() { return val; }
  }
  private final String INIT_VAL = "initdisps/initdisp[{0}]/{1}";
  private final String INITS = "initdisps/initdisp";
  public enum RETRIEVE_TAG {
    ID ("id"),
    DISP("display"),
    PROC("process"),
    ACT("action"),
    OPT("option");
    private final String val;
    RETRIEVE_TAG(String argval) { val = argval; }
    public String getVal() { return val; }
  }
  private final String RETRIEVE_VAL = "retrieves/retrieve[{0}]/{1}";
  private final String RETRIEVES = "retrieves/retrieve";

  private static final String FILE_NAME = "plugininfo.xml";
  private Element _infXml = null;
  /**
   * Describe <code>setXml</code> method : setting information xml Document.
   * <br>w肳ꂽtH_[̃vOCXMLݒ肷B
   *
   * @param flnm a <code>String</code> :
   * @throws Exception
   */
  public void setXml(File dir) throws Exception {
    setXml(dir.getPath());
  }
  /**
   * Describe <code>setXml</code> method : setting information xml Document.
   * <br>w肳ꂽt@CvOCXMLݒ肷B
   *
   * @param filePass a <code>String</code> : vOCXML̃pX
   * @throws Exception
   */
  public void setXml(String filePass) throws Exception {
    StringBuilder flnm = new StringBuilder();
    flnm.append(filePass);
    flnm.append(File.separator);
    flnm.append(FILE_NAME);
    _infXml = XmlUtil.createDocument(flnm.toString()).getDocumentElement();
  }

  /**
   * Describe <code>getInfTag</code> method : gettin tag value.
   * <br>w肳ꂽTag̒l擾B
   *
   * @param tag a <code>INF_TAG</code> : Tag\enum
   * @return a <code>String</code> : 擾l
   */
  public String getInfTag(INF_TAG tag) {
    if (_infXml == null) return null;
    return XPathUtil.getValueWithPath(_infXml, tag.toString());
  }

  /**
   * Describe <code>getMenuCount</code> method : get menu tag counter.
   * <br>"menu"^O̐߂B
   * <br>
   * @return a <code>int</code> : menu tag count
   */
  public int getMenuCount() {
  	NodeList ndlMenu = XPathUtil.searchElement(_infXml, MENUS);
  	if (ndlMenu == null) return 0;
  	return ndlMenu.getLength();
  }

  /**
   * Describe <code>getMenuVal</code> method : get tag in menu value.
   * <br>menu^Õ^O̒l擾B
   * <br>
   * @param tag a <code>MENU_TAG</code> :
   * @param pos an <code>int</code> :
   * @return a <code>String</code> :
   */
  public String getMenuVal(MENU_TAG tag, int pos) {
  	Object[] parm = {String.valueOf(pos), tag.getVal()};
  	String path = MessageFormat.format(MENU_VAL, parm);
  	return XPathUtil.getValueWithPath(_infXml, path);
  }

  /**
   * Describe <code>getInitCount</code> method : get initdisp tag counter.
   * <br>"initdisp"^O̐߂B
   * <br>
   * @return a <code>int</code> : initdisp tag count
   */
  public int getInitCount() {
    NodeList ndlInit = XPathUtil.searchElement(_infXml, INITS);
    if (ndlInit == null) return 0;
    return ndlInit.getLength();
  }

  /**
   * Describe <code>getInitVal</code> method : get tag value in initdisp.
   * <br>initdisp^Õ^O̒l擾B
   * <br>
   * @param tag a <code>INIT_TAG</code> :
   * @param pos an <code>int</code> :
   * @return a <code>String</code> :
   */
  public String getInitVal(INIT_TAG tag, int pos) {
    Object[] parm = {String.valueOf(pos), tag.getVal()};
    String path = MessageFormat.format(INIT_VAL, parm);
    return XPathUtil.getValueWithPath(_infXml, path);
  }

  /**
   * Describe <code>getRetrieveCount</code> method : get retrieve tag counter.
   * <br>"retrieve"^O̐߂B
   *
   * @return a <code>int</code> : retrieve tag count
   */
  public int getRetrieveCount() {
    NodeList ndlRetrieve = XPathUtil.searchElement(_infXml, RETRIEVES);
    if (ndlRetrieve == null) return 0;
    return ndlRetrieve.getLength();
  }

  /**
   * Describe <code>getRetrieveVal</code> method : get tag value in retrive.
   * <br>retrive^ÕR^O̒l߂B
   *
   * @param tag a <code>RETRIEVE_TAG</code> :
   * @param pos an <code>int</code> :
   * @return a <code>String</code> :
   */
  public String getRetrieveVal(RETRIEVE_TAG tag, int pos) {
    Object[] parm = {String.valueOf(pos), tag.getVal()};
    String path = MessageFormat.format(RETRIEVE_VAL, parm);
    return XPathUtil.getValueWithPath(_infXml, path);
  }
}
