package com.hamaf.hikyaku.report.excel2003;

import java.io.OutputStream;
import java.io.Writer;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import com.hamaf.hikyaku.util.XmlUtil;

/**
 * <p>^CgF ExcelDoc is Microsoft Excel SpredsheetML Object.</p>
 * <p>FExcelpspredsheetML(XML Document)ǗB</p>
 * <br>ȉ̋@\񋟂
 * <ul>
 *   <li>openBook : SpreadSheetMLt@C擾
 *   <li>newBook : VKSpreadSheetML쐬
 *   <li>getSheet : [NV[g擾
 *   <li>newSheet : V[NV[gǉ
 *   <li>getExcelStyle : X^CݒNX擾
 *   <li>addStyle : X^Cǉ
 *   <li>getExldoc : hLg߂
 *   <li>getStyles : X^CEGg擾
 *   <li>writeDoc : hLgo͂
 * </ul>
 *
 *
 * <p>Created: Tue Nov 25 11:43:04 2008</p>
 * @author y-hamanaka - <i>hamanaka</i>
 * @version 1.0
 */
public class ExcelDoc {

  public static final String STYL_DATE = "datedef";

  private Document _exldoc = null;
  private Element _book = null;
  private Element _style = null;
  private int _styNo = 0;

  private static final String MSO = "mso-application";
  private static final String PROG = "progid=\"Excel.Sheet\"";
  public ExcelDoc() {
  }

  /**
   * Describe <code>openBook</code> method : open Excel spreadsheetMl file.
   * <br>Excelp SpreadSheetMLt@CJ
   *
   * @param flnm a <code>String</code> : ΏۂƂXMLt@C
   * @return a <code>Document</code> : 擾XML Document
   * @exception Exception if an error occurs
   */
  public Document openBook(String flnm) throws Exception {
    _exldoc = XmlUtil.createDocument(flnm);
    _book = _exldoc.getDocumentElement();
    XPath xp = XPathFactory.newInstance().newXPath();
    Node sty = (Node)xp.evaluate("Styles", _book, XPathConstants.NODE);
    if (sty != null) {
      _style = (Element)sty;
      _style.appendChild(createDateStyle());
    } else {
      createStyle();
      _book.appendChild(_style);
    }
    return _exldoc;
  }
  
  /**
   * Describe <code>newBook</code> method : create Excel Wookbook SpredsheetML.
   * <br>Microsoft Excel workbook 쐬B
   * @return a <code>Document</code> : Workbook XML
   * @exception Exception if an error occurs
   */
  public Document newBook() throws Exception {
    _exldoc = XmlUtil.createDocument();
    _exldoc.appendChild(_exldoc.createProcessingInstruction(MSO, PROG));
    _book = createWorkbook();
    createStyle();
    _book.appendChild(_style);
    _exldoc.appendChild(_book);
    return _exldoc;
  }
  
  private static final String NS = "http://www.w3.org/2000/xmlns/";
  private static final String SHT = "urn:schemas-microsoft-com:office:spreadsheet";
  private static final String EXL = "urn:schemas-microsoft-com:office:excel";
  private Element createWorkbook() throws Exception {
    Element wbook = _exldoc.createElement("Workbook");
    wbook.setAttributeNS(NS,"xmlns",SHT);
    wbook.setAttributeNS(NS, "xmlns:x", EXL);
    wbook.setAttributeNS(NS,"xmlns:ss",SHT);
    return wbook;
  }
  private void createStyle() throws Exception {
    _style = _exldoc.createElement("Styles");
    Element def = _exldoc.createElement("Style");
    def.setAttribute("ss:ID", "Default");
    def.setAttribute("ss:Name","Normal");
    Element alm = _exldoc.createElement("Alignment");
    alm.setAttribute("ss:Vertical","Center");
    def.appendChild(alm);
    Element fnt = _exldoc.createElement("Font");
    fnt.setAttribute("ss:FontName","lr oSVbN");
    fnt.setAttribute("x:CharSet","128");
    fnt.setAttribute("x:Family","Modern");
    fnt.setAttribute("ss:Size","11");
    def.appendChild(fnt);
    _style.appendChild(def);

    _style.appendChild(createDateStyle());
  }
  private Element createDateStyle() throws Exception {
    Element dtdef = _exldoc.createElement("Style");
    dtdef.setAttribute("ss:ID", STYL_DATE);
    Element dtcld = _exldoc.createElement("NumberFormat");
    dtcld.setAttribute("ss:Format", "Short Date");
    dtdef.appendChild(dtcld);
    return dtdef;
  }

  /**
   * Describe <code>getSheet</code> method : get worksheet by name.
   * <br>w肳ꂽ̂̃[NV[g擾
   *
   * @param name a <code>String</code> : [NV[g
   * @return an <code>ExcelSheet</code> : 擾[NV[g
   * @exception Exception if an error occurs
   */
  public ExcelSheet getSheet(String name) throws Exception {
    return getExSht("Worksheet[@ss:Name='" + name + "']");
  }
  /**
   * Describe <code>getSheet</code> method : get worksheet at position.
   * <br>w肳ꂽʒu(0Ԗڂ)̃[NV[g擾
   *
   * @param pos an <code>int</code> : [NV[g̈ʒu ('0'Ԗڂw肷)
   * @return an <code>ExcelSheet</code> : 擾[NV[g
   * @exception Excepsion if an error occurs
   */
  public ExcelSheet getSheet(int pos) throws Exception {
    int p = pos;
    if (p < 0) p = 0;
    p++;
    return getExSht("Worksheet[" + p + "]");
  }
  private ExcelSheet getExSht(String searchVal) throws Exception {
    XPath xp = XPathFactory.newInstance().newXPath();
    Element sht = (Element)xp.evaluate(searchVal, _book, XPathConstants.NODE);
    if (sht == null) {
      throw new IllegalAccessException("w肳ꂽV[g " + searchVal + " ݂͑܂B");
    }

    ExcelSheet es = new ExcelSheet();
    es.init(this, sht);
    return es;
  }
  /**
   * Describe <code>newSheet</code> method : create new worksheet.
   * <br>[NV[g𐶐AbookɒǉB
   *
   * @param name a <code>String</code> : [NV[g
   * @return an <code>ExcelSheet</code> : [NV[g
   * @exception Exception if an error occurs
   */
  public ExcelSheet newSheet(String name) throws Exception {
    ExcelSheet es = new ExcelSheet();
    es.init(this, name);
    _book.appendChild(es.getSheet());
    return es;
  }

  /**
   * Describe <code>writeDoc</code> method : t@CɃhLgo͂.
   *
   * @param flnm a <code>String</code> : o̓t@C
   * @param encd a <code>String</code> : R[h(l "utf-8")
   * @exception Exception if an error occurs
   */
  public void writeDoc(String flnm, String encd) throws Exception {
    XmlUtil.writeXml(_exldoc, flnm, encd);
  }
  /**
   * Describe <code>writeDoc</code> method : OutputStreamɃhLgo͂.
   *
   * @param out an <code>OutputStream</code> : o͐
   * @param encd a <code>String</code> : R[h(l "utf-8")
   * @exception Exception if an error occurs
   */
  public void writeDoc(OutputStream out, String encd) throws Exception {
    XmlUtil.writeXml(_exldoc, out, encd);
  }
  /**
   * Describe <code>writeDoc</code> method : WriterɃhLgo͂.
   * <br>R[hɑΉ<code>Writer</code>w肷邱ƁB
   *
   * @param out a <code>Writer</code> : o͐
   * @param encd a <code>String</code> : R[h(l "utf-8")
   * @exception Exception if an error occurs
   */
  public void writeDoc(Writer out, String encd) throws Exception {
    XmlUtil.writeXml(_exldoc, out, encd);
  }

  /**
   * Gets the value of exlDoc
   *
   * @return the value of exlDoc
   */
  public final Document getExldoc() {
    return this._exldoc;
  }

  /**
   * Gets the value of style
   *
   * @return the value of style
   */
  public final Element getStyles() {
    return this._style;
  }

  /**
   * Describe <code>getExcelStyle</code> method : X^CݒNX𐶐.
   *
   * @return an <code>ExcelStyle</code> : 
   */
  public final ExcelStyle getExcelStyle() {
    return new ExcelStyle(_exldoc);
  }
  
  /**
   * Describe <code>addStyle</code> method : X^Cǉ.
   *
   * @param sty an <code>ExcelStyle</code> : 
   * @return a <code>String</code> : ̃X^CID
   * @exception Exception if an error occurs
   */
  public final String addStyle(ExcelStyle sty) throws Exception {
    _styNo++;
    String styId =  "myst" + _styNo;
    _style.appendChild(sty.getStyle(styId));
    return styId;
  }
}
