package com.hamaf.hikyaku.report.excel2003;

import java.util.ArrayList;
import java.util.List;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * <p>^CgF ExcelSheet is Excel sheet object.</p>
 * <p>FExcel̃V[gɑΉIuWFNgB</p>
 * <br>ȉ̋@\񋟂
 * <ol>
 *   <li>init(ExcelDoc, String) : iVK쐬pj
 *   <li>init(ExcelDoc, Element) : Ggێ
 *   <li>setWidth : eZݒ肷
 *   <li>getValue : Z̒l擾
 *   <li>setValue : w͈͂̊eZ̒lݒ肷
 *   <li>setRowValue : 1s̃Z̒lݒ肷
 *   <li>setCellValue : 1Z̒lݒ肷
 *   <li>copyAddValue : 1sʂAlݒ肷
 *   <li>getRow : sIuWFNg擾
 *   <li>getRowCount : s擾
 *   <li>deleteRow : s폜
 *   <li>setStyle : w͈͂̊eZɃX^Cݒ肷
 *   <li>getSheet : V[gEGg߂
 * </ol>
 *
 *
 * <p>Created: Tue Nov 25 14:35:53 2008</p>
 * @author y-hamanaka - <i></i>
 * @version 1.0
 */
public class ExcelSheet {

  private static final String IDX = "ss:Index";
  private static final String DEF_WDH = "54";
  private static final String DEF_HIGH = "13.5";

  private final List<String> TBL_ATR = new ArrayList<String>();
  {
    TBL_ATR.add("ss:ExpandedColumnCount");
    TBL_ATR.add("ss:ExpandedRowCount");
    TBL_ATR.add("x:FullColumns");
    TBL_ATR.add("x:FullRows");
  };
  
  private ExcelDoc _exl = null;
  private Document _doc = null;
  private Element _sheet = null;
  private Element _tbl = null;
  private XPath _xp = null;
  private XPathExpression _getRow = null;

  /**
   * Creates a new <code>ExcelSheet</code> instance.
   *
   */
  public ExcelSheet() {
    _xp = XPathFactory.newInstance().newXPath();
    try {
      _getRow = _xp.compile("Row");
    } catch (Exception xe) {
      xe.printStackTrace();
    }
  }
  /**
   * Describe <code>init</code> method : create WorkSheet Element.
   * <br>WorkSheetɑΉGg𐶐B
   *
   * @param book an <code>ExcelDoc</code> : 
   * @param name a <code>String</code> : V[g
   * @exception Exception if an error occurs
   */
  public void init(ExcelDoc book, String name) throws Exception {
    if (_sheet != null) {
      return;
    }
    _exl = book;
    _doc = book.getExldoc();
    _sheet = _doc.createElement("Worksheet");
    _sheet.setAttribute("ss:Name",name);
    _tbl = _doc.createElement("Table");
    _tbl.setAttribute("ss:DefaultColumnWidth", DEF_WDH);
    _tbl.setAttribute("ss:DefaultRowHeight", DEF_HIGH);
    _sheet.appendChild(_tbl);
  }

  /**
   * Describe <code>init</code> method : save worksheet Element.
   * <br>̃[NV[gEGgێ
   *
   * @param book an <code>ExcelDoc</code> : 
   * @param wsht an <code>Element</code> : 
   */
  public void init(ExcelDoc book, Element wsht) throws Exception {
    _exl = book;
    _doc = book.getExldoc();
    _sheet = wsht;
    _tbl = (Element)_xp.evaluate(".//Table", wsht, XPathConstants.NODE);
    checkRow();
  }
  private void checkRow() throws Exception {
    NodeList rows = (NodeList)_getRow.evaluate(_tbl, XPathConstants.NODESET);
    int p = 0;
    for (int l = 0; l < rows.getLength(); l++) {
      p++;
      Element rowe = (Element)rows.item(l);
      ExcelRow roww = new ExcelRow();
      roww.init(rowe);
      String sln = rowe.getAttribute(IDX);
      if (sln.length() > 0) {
        int ix = Integer.parseInt(sln);
        for (int j = p; j < ix; j++) {
          ExcelRow xrow = new ExcelRow();
          xrow.init(_doc, "");
          _tbl.insertBefore(xrow.getRow(), rowe);
          p++;
        }
        rowe.removeAttribute(IDX);
      }
    }
    for (String atr : TBL_ATR) {
      _tbl.removeAttribute(atr);
    }
  }
  
  /**
   * Describe <code>setWidth</code> method : setting columns width.
   * <br>eJ̕ݒ肷B
   * <br>Excel"̎w"Őݒ肷leJɎw肷B
   * <br>ύXȂJɑ΂Ă͋̕("")w肷邱ƁB
   *
   * @param width a <code>List&lt;String&gt;</code> : eJ̕w肵Xg
   * @exception Exception if an error occurs
   */
  public void setWidth(List<String> width) throws Exception {
    delColumns();
    Node row = (Node)_xp.evaluate("Row[1]", _tbl, XPathConstants.NODE);
    int p = 0;
    for (String ln : width) {
      p++;
      if (ln != null && ln.length() > 0 && !ln.equals(DEF_WDH)) {
        Element col = _doc.createElement("Column");
        col.setAttribute(IDX, String.valueOf(p));
        col.setAttribute("ss:AutoFitWidth", "0");
        col.setAttribute("ss:Width", convWidth(ln));
        if (row == null) {
          _tbl.appendChild(col);
        } else {
          _tbl.insertBefore(col, row);
        }
      }
    }
  }
  private String convWidth(String org) {
    float fval = Float.parseFloat(org);
    float ret = (((8 * fval) + 5) * 3 ) / 4;
    int sei = (int)ret;
    int syo = Math.round((ret - sei) * 100);
    StringBuilder sb = new StringBuilder();
    sb.append(sei);
    if (syo > 0) {
      sb.append(".");
      sb.append(syo);
    }
    return sb.toString();
  }
  private void delColumns() throws Exception {
    if (!_tbl.hasChildNodes()) {
      return;
    }
    XPath xp = XPathFactory.newInstance().newXPath();
    NodeList ndl = (NodeList)xp.evaluate("Column", _tbl,
                                         XPathConstants.NODESET);
    for (int i = 0; i < ndl.getLength(); i++) {
      _tbl.removeChild(ndl.item(i));
    }
  }

  /**
   * Describe <code>getValue</code> method : get cell value.
   * <br>w肳ꂽAsA̒l擾
   *
   * @param row an <code>int</code> : sʒu
   * @param col an <code>int</code> : ʒu
   * @return a <code>String</code> : 擾l
   */
  public String getValue(int row, int col) {
    try {
      Element relm = (Element)_xp.evaluate("Row[" + row + "]", _tbl, XPathConstants.NODE);
      if (relm != null) {
        ExcelRow xrow = new ExcelRow();
        xrow.init(relm);
        return xrow.getValue(col);
      }
    } catch (Exception  xe) {
      xe.printStackTrace();
    }
    return "";
  }
  
  /**
   * Describe <code>setValue</code> method : set valus.
   * <br>[NV[gɒlݒ肷B
   * <br>l͍s̃XgɊe񖈂̒lXgݒ肷B
   * <br>ws(row)̒l "0" ̏ꍇ́AǉƂȂB
   *
   * @param val a <code>List&lt;List&lt;Object&gt;&gt;</code> : ݒ肷lXg
   * @param row an <code>int</code> : Jns
   * @param col an <code>int</code> : Jn
   * @param high a <code>String</code> : s̍
   * @exception Exception if an error occurs
   */
  public void setValue(List<List<Object>> val,
                       int row, int col, String high) throws Exception {
    NodeList rws = (NodeList)_getRow.evaluate(_tbl, XPathConstants.NODESET); 
    int max = rws.getLength();
    if (row < 1 || max == 0 || row > max) {
      for (int i = max + 1; i < row; i++) {
        addNullRow();
      }
      addRow(val, high, col);
      return;
    }

    Element base = (Element)rws.item(max);
    for (List<Object> rval : val) {
      if (row > max) {
        copyRow(base, rval, col);
      } else {
        resetRow(row, rval, col);
      }
      row++;
    }
  }
  /**
   * Describe <code>setRowValue</code> method : 1s̒lݒ肷.
   *
   * @param val a <code>List&lt;Object&gt</code> : ݒ肷lXg
   * @param row an <code>int</code> : ݒsʒu
   * @param col an <code>int</code> : Jn
   * @param high a <code>String</code> : s̍
   * @exception Exception if an error occurs
   */
  public void setRowValue(List<Object> val,
                          int row, int col, String high) throws Exception {
    List<List<Object>> vals = new ArrayList<List<Object>>();
    vals.add(val);
    setValue(vals, row, col, high);
  }
  /**
   * Describe <code>setCellValue</code> method : PZ̒lݒ肷.
   *
   * @param val an <code>Object</code> : ݒ肷l
   * @param row an <code>int</code> : ݒsʒu
   * @param col an <code>int</code> : ݒʒu
   * @param high a <code>String</code> : s̍
   * @exception Exception if an error occurs
   */
  public void setCellValue(Object val,
                          int row, int col, String high) throws Exception {
    List<List<Object>> vals = new ArrayList<List<Object>>();
    List<Object> dt = new ArrayList<Object>();
    dt.add(val);
    vals.add(dt);
    setValue(vals, row, col, high);
  }

  /**
   * Describe <code>copyAddValue</code> method : row copy and value set.
   * <br>w肳ꂽsɕʂAlݒ肷B
   *
   * @param val a <code>List&lt;List&lt;Object&gt;&gt;</code> : ݒ肷lXg
   * @param org an <code>int</code> : ɂsʒu 
   * @param col an <code>int</code> : ݒʒu
   * @exception Exception if an error occurs
   */
  public void copyAddValue(List<List<Object>> val, int org, int col) throws Exception {
    if (org < 1) {
      throw new IllegalArgumentException("ws1ȏłȂ΂܂B wl:" + org);
    }
    NodeList rws = (NodeList)_getRow.evaluate(_tbl, XPathConstants.NODESET);
    int max = rws.getLength();
    if (org > max || max == 0) {
      throw new IllegalAccessException("w肳ꂽs " + org + " ݂͑܂B");
    }
    Element orgRow = (Element)rws.item(org - 1);
    int l = val.size();
    for (List<Object> rval : val) {
      l--;
      if (l == 0) {
        resetRow(org + val.size() - 1, rval, col);
      } else {
        copyRow(orgRow, rval, col);
      }
    }
  }

  /**
   * Describe <code>getRow</code> method : get Row Object.
   * <br>ExcelsIuWFNg擾
   *
   * @param row an <code>int</code> : sʒu
   * @return an <code>ExcelRow</code> : 擾Excel Row IuWFNg
   * @exception Exception if an error occurs
   */
  public ExcelRow getRow(int row) throws Exception {
    Element org = (Element)_xp.evaluate("Row[" + row + "]", _tbl,
                                        XPathConstants.NODE);
    if (org == null) {
      return null;
    }
    
    ExcelRow xrow = new ExcelRow();
    xrow.init(org);
    return xrow;
  }

  /**
   * Describe <code>getRowCount</code> method : get row count.
   * <br>[NV[gɐݒ肳Ăs擾
   *
   * @return an <code>int</code> : s 
   */
  public int getRowCount() {
    try {
      NodeList rws = (NodeList)_getRow.evaluate(_tbl, XPathConstants.NODESET); 
      return rws.getLength();
    } catch (Exception xe) {
      xe.printStackTrace();
    }
    return 0;
  }
  
  /**
   * Describe <code>deleteRow</code> method : delete row.
   * <br>w肳ꂽs폜A폜Node߂
   *
   * @param row an <code>int</code> : 폜sʒu
   * @return a <code>Node</code> : 폜Node
   * @exception Exception if an error occurs
   */
  public Node deleteRow(int row) throws Exception {
    Node org = (Node)_xp.evaluate("Row[" + row + "]", _tbl, XPathConstants.NODE);
    if (org == null) {
      throw new IllegalAccessException("w肳ꂽs " + row + " ݂͑܂B");
    }
    return _tbl.removeChild(org);
  }

  private static final String SR_CELL_B = ".//Cell[Data='";
  private static final String SR_CELL_A = "']";
  /**
   * Describe <code>searchCell</code> method : search cell position by value.
   * <br>w肳ꂽlŹ@sA 擾B
   * 
   * @param val a <code>String</code> : l
   * @return a <code>int[]</code> : [s, ]̒l
   * @throws Exception
   */
  public int[] searchCell(String val) throws Exception {
    int[] ret = new int[2];
    StringBuilder sb = new StringBuilder();
    sb.append(SR_CELL_B);
    sb.append(val);
    sb.append(SR_CELL_A);
    Node cell = (Node)_xp.evaluate(sb.toString(), _tbl, XPathConstants.NODE);
    ret[1] = getPos(cell);
    ret[0] = getPos(cell.getParentNode());
    return ret;
  }

  /**
   * Describe <code>setStyle</code> method : w肳ꂽ͈͂ɃX^Cݒ肷.
   *
   * @param styId a <code>String</code> : X^Cl
   * @param strRow an <code>int</code> : s̊Jnʒu
   * @param endRow an <code>int</code> : s̏Iʒu
   * @param strCol an <code>int</code> : ̊Jnʒu
   * @param endCol an <code>int</code> : ̏Iʒu
   * @exception Exception if an error occurs
   */
  public void setStyle(String styId, int strRow, int endRow,
                       int strCol, int endCol) throws Exception {
    for (int p = strRow; p <= endRow; p++) {
      ExcelRow er = getRow(p);
      if (er != null) {
        er.setStyle(styId, strCol, endCol);
      }
    }
  }
  /**
   * Describe <code>setStyle</code> method : w肳ꂽ͈͂ɃX^Cݒ肷.
   *
   * @param sty an <code>ExcelStyle</code> : 
   * @param strRow an <code>int</code> : s̊Jnʒu
   * @param endRow an <code>int</code> : s̏Iʒu
   * @param strCol an <code>int</code> : ̊Jnʒu
   * @param endCol an <code>int</code> : ̏Iʒu
   * @exception Exception if an error occurs
   */
  public void setStyle(ExcelStyle sty, int strRow, int endRow,
                       int strCol, int endCol) throws Exception {
    String id = _exl.addStyle(sty);
    setStyle(id, strRow, endRow, strCol, endCol);
  }

  private int getPos(Node chld) {
    Node cnd = chld.getParentNode().getFirstChild();
    int p = 0;
    while (cnd != null) {
      if (cnd.getNodeType() == Node.ELEMENT_NODE &&
          (cnd.getNodeName().equals("Row")
          || cnd.getNodeName().equals("Cell"))) {
        p++;
        String idx = ((Element)cnd).getAttribute(IDX);
        if (idx != null && idx.length() > 0) {
          p = Integer.parseInt(idx);
        }
        if (chld.isEqualNode(cnd)) {
          return p;
        }
      }
      cnd = cnd.getNextSibling();
    }
    return 0;
  }
  private void resetRow(int row, List<Object> rval, int col) throws Exception {
    ExcelRow xrow = getRow(row);
    xrow.setValues(rval, col);
  }
  private void copyRow(Element base, List<Object> rval, int col) throws Exception {
    Element newRow = (Element)base.cloneNode(true);
    _tbl.insertBefore(newRow, base);
    ExcelRow xrow = new ExcelRow();
    xrow.init(newRow);
    xrow.setValues(rval, col);
  }
  private void addNullRow() throws Exception {
    ExcelRow xrow = new ExcelRow();
    xrow.init(_doc, "");
    _tbl.appendChild(xrow.getRow());
  }
  private void addRow(List<List<Object>> vals, String high, int col) throws Exception {
    for (List<Object> rval : vals) {
      ExcelRow xrow = new ExcelRow();
      xrow.init(_doc, high);
      xrow.setValues(rval, col);
      _tbl.appendChild(xrow.getRow());
    }
  }
  
  /**
   * Gets the value of sht
   *
   * @return the value of sht
   */
  public final Element getSheet() {
    return this._sheet;
  }

}
