package com.hamaf.hikyaku.security.process;

import java.io.File;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import javax.servlet.http.HttpSession;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.USE_FLG;
import com.hamaf.hikyaku.distribute.DistributeServlet;
import com.hamaf.hikyaku.distribute.ServletUtils;
import com.hamaf.hikyaku.security.access.GroupUserSql;
import com.hamaf.hikyaku.security.access.UserAccess;
import com.hamaf.hikyaku.security.access.UserAccess.PWD_FLG;
import com.hamaf.hikyaku.security.access.UserAccess.USER_TYPE;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.security.dto.UserPwdDto;
import com.hamaf.hikyaku.security.dto.UserSearchDto;
import com.hamaf.hikyaku.security.ui.LogInCheck;
import com.hamaf.hikyaku.security.ui.UserIconMod;

/**
 * <p>^CgFUserProcess is User Master process.</p>
 * <p>F[U[̏s</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2010/03/16</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class UserProcess {

  private UserAccess _ua = null;
  private GroupUserSql _gus = null;
  /**
   *  Creates a new <code>UserProcess.java</code> instance.
   * <br>Default Constructor
   */
  public UserProcess(UserAccess ua, GroupUserSql gus) {
    _ua = ua;
    _gus = gus;
  }

  /**
   * Describe <code>find</code> method : .
   * <br>[U[̌B
   *
   * @param usd a <code>UserSearchDto</code> :
   * @return a <code>UserSearchDto</code> :
   * @throws Exception
   */
  public UserSearchDto find(UserSearchDto usd) throws Exception {
    return (UserSearchDto)_ua.find(usd);
  }

  /**
   * Describe <code>findByPrimarykey</code> method : .
   * <br>[U[ڍ׏̎擾B
   *
   * @param ud a <code>UserDto</code> :
   * @return a <code>UserDto</code> :
   * @throws Exception
   */
  public UserDto findByPrimarykey(UserDto ud, HttpSession hs) throws Exception {
    Object udt = hs.getAttribute(LogInCheck.USER_INF);
    if (udt != null && udt instanceof UserDto) {
      ud.setCommFld(COMMFLD.UUSR, ((UserDto)udt).getItemValue(UserDto.USER_ID));
    }
    return (UserDto)_ua.findByPrimaryKey(ud);
  }

  /**
   * Describe <code>getDetailUserById</code> method : .
   * <br>[U[IDŃ[U[擾B
   *
   * @param uid a <code>String</code> : [U[ID
   * @return a <code>UserDto</code> :
   * @throws Exception
   */
  public UserDto getDetailUserById(String uid, HttpSession hs) throws Exception {
    UserDto ud = new UserDto();
    ud.setItemValue(UserDto.USER_ID, uid);
    Object udt = hs.getAttribute(LogInCheck.USER_INF);
    if (udt != null && udt instanceof UserDto) {
      ud.setCommFld(COMMFLD.UUSR, ((UserDto)udt).getItemValue(UserDto.USER_ID));
    } else {
      ud.setCommFld(COMMFLD.UUSR, hs.getId());
    }
    ud.setAllRequest();
    ud.setSysId(DistributeServlet.SYSID);
    return (UserDto)_ua.findByPrimaryKey(ud);
  }

  /**
   * Describe <code>getUserPwdInf</code> method : .
   * <br>[U[IDŃpX[h̏擾B
   *
   * @param uid a <code>String</code> : [U[ID
   * @return a <code>UserPwdDto</code> :
   * @throws Exception
   */
  public UserPwdDto getUserPwdInf(String uid, HttpSession hs) throws Exception {
    UserPwdDto ud = new UserPwdDto();
    ud.setItemValue(UserPwdDto.USER_ID, uid);
    ud.setCommFld(COMMFLD.DEL, USE_FLG.USE.getVal());
    Object udt = hs.getAttribute(LogInCheck.USER_INF);
    if (udt != null && udt instanceof UserDto) {
      ud.setCommFld(COMMFLD.UUSR, ((UserDto)udt).getItemValue(UserDto.USER_ID));
    } else {
      ud.setCommFld(COMMFLD.UUSR, hs.getId());
    }
    ud.setAllRequest();
    ud.setSysId(DistributeServlet.SYSID);
    return (UserPwdDto)_ua.findByPrimaryKey(ud);
  }

  /**
   * Describe <code>update</code> method : .
   * <br>[U[̍XVB
   *
   * @param ud a <code>UserDto</code> :
   * @throws Exception
   */
  public void update(UserDto ud) throws Exception {
    ud.setCommFld(COMMFLD.UPGM, this.getClass().getName());
    _ua.update(ud);
  }

  /**
   * Describe <code>updatePassword</code> method : .
   * <br>pX[h̍XVB
   *
   * @param uid a <code>String</code> : [U[ID
   * @param pwd a <code>String</code> : pX[h
   * @param init a <code>String</code> : tO
   * @throws Exception
   */
  public void updatePassword(String uid, String pwd, PWD_FLG pf, HttpSession hs) throws Exception {
    UserPwdDto uad = getUserPwdInf(uid, hs);
    uad.setItemValue(UserPwdDto.OLD_PWD3, uad.getItemValue(UserPwdDto.OLD_PWD2));
    uad.setItemValue(UserPwdDto.OLD_PWD2, uad.getItemValue(UserPwdDto.OLD_PWD1));
    uad.setItemValue(UserPwdDto.OLD_PWD1, uad.getItemValue(UserPwdDto.PASSWORD));
    uad.setItemValue(UserPwdDto.PASSWORD, PasswordUtil.encrypt(pwd));
    DateFormat dfd = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.JAPAN);
    uad.setItemValue(UserPwdDto.PWD_UPDATE, dfd.format(new Date()));
    uad.setItemValue(UserPwdDto.INITIALIZE_FLG, pf.getVal());
    uad.setItemValue(UserPwdDto.ERROR_COUNT, "0");

    uad.setCommFld(COMMFLD.UPGM, this.getClass().getName());
    uad.setSysId(DistributeServlet.SYSID);
    _ua.update(uad);
  }

  public void setErrorCount(String uid, int cnt, HttpSession hs) throws Exception {
    UserPwdDto uad = getUserPwdInf(uid, hs);
    uad.setItemValue(UserPwdDto.ERROR_COUNT, String.valueOf(cnt));
    uad.setCommFld(COMMFLD.UPGM, this.getClass().getName());
    _ua.update(uad);
  }

  /**
   * Describe <code>insert</code> method : .
   * <br>[U[̓o^B
   *
   * @param ud a <code>UserDto</code> :
   * @throws Exception
   */
  public void insert(UserDto ud) throws Exception {
    ud.setItemValue(UserDto.USER_TYPE, USER_TYPE.GENERAL.getVal());
    ud.setItemValue(UserDto.INITIALIZE_FLG, PWD_FLG.EMPTY.getVal());
    ud.setCommFld(COMMFLD.UPGM, this.getClass().getName());
    _ua.regist(ud);
  }

  /**
   * Describe <code>delete</code> method : .
   * <br>[U폜B
   *
   * @param ud a <code>UserDto</code> :
   * @throws Exception
   */
  public void delete(UserDto ud) throws Exception {
    String uid = ud.getItemValue(UserDto.USER_ID);
    String icon = ud.getItemValue(UserDto.ICON_UMU);
    _ua.delete(ud);
//    ud.setCommFld(COMMFLD.DEL, USE_FLG.DEL.getVal());
//    _ua.update(ud);

    PreparedItems pi = new PreparedItems();
    pi.setSysId(DistributeServlet.SYSID);
    Map<String, String> vals = new HashMap<String, String>();
    vals.put("UserId", uid);
    pi.addParam(vals);
    _gus.deleteByUserId(pi);

    StringBuilder sb = new StringBuilder();
    sb.append(ServletUtils.whichFile(UserIconMod.ICON_PATH));
    sb.append("/");
    sb.append(uid);
    sb.append(".");
    sb.append(icon);
    File iconf = new File(sb.toString());
    if (iconf.exists()) iconf.delete();
  }
}
