package com.hamaf.hikyaku.security.ui;

import java.util.Map;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.extension.MasterXmlControl;
import com.hamaf.hikyaku.extension.MasterXmlFactory;
import com.hamaf.hikyaku.security.access.UserAccess.PWD_FLG;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.security.dto.UserPwdDto;
import com.hamaf.hikyaku.security.process.PasswordUtil;
import com.hamaf.hikyaku.security.process.UserProcess;
import com.hamaf.hikyaku.ui.StandardCheckAndUpdate;

/**
 * <p>^CgFPasswordChange is .</p>
 * <p>F</p>
 * <br>
 * 
 * <p>쌠: Copyright (c) 2010/03/24</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class PasswordChange extends StandardCheckAndUpdate {

  private UserProcess _up = null; 
  /**
   *  Creates a new <code>PasswordChange.java</code> instance.
   * <br>Default Constructor
   */
  public PasswordChange(UserProcess up) {
    _jim = null;
    _up = up;
  }
  
  private final String PWD = "new_password";
  private final String CPWD = "conf_password";

  private final String DOUITU = "PwdErrMatch";
  private final String UNMATCH = "PwdErrUnMatch";
  private final String FMT_ERR = "PwdErrFormat";
  private final String PRV_ERR = "PwdErrPrevious";
  private final String MX_TAG = "password";
  private final String MX_PRE = "previous";
  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#checkRelation(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage checkRelation(CommonMessage msg) throws Exception {
    String npwd = msg.getReqStr(PWD);
    String conf = msg.getReqStr(CPWD);
    String usrid = msg.getReqStr(PasswordChangeIni.MSG_UID);
    if (!npwd.equals(conf)) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.addMessage(resource.getString(UNMATCH));
      return msg;
    }
    
    if (PasswordUtil.checkChar(npwd)) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.addMessage(resource.getString(FMT_ERR));
      return msg;
    }
    
    String err = chackPrevious(usrid, npwd);
    if (err != null) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.addMessage(resource.getString(err));
      return msg;
    }
    
    return super.checkRelation(msg);
  }
  
  private String chackPrevious(String uid, String pwd) throws Exception {
    UserPwdDto upd = _up.getUserPwdInf(uid, session);
    String nowpwd = PasswordUtil.decrypt(upd.getItemValue(UserPwdDto.PASSWORD));
    if (nowpwd.equals(pwd)) return DOUITU;
    
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(sysId);
    Map<String, String> dt = mxc.getItemValue(MX_TAG, "cd");
    if ("yes".equals(dt.get(MX_PRE))) {
      String[] oldpwd = new String[3];
      oldpwd[0] = PasswordUtil.decrypt(upd.getItemValue(UserPwdDto.OLD_PWD1));
      oldpwd[1] = PasswordUtil.decrypt(upd.getItemValue(UserPwdDto.OLD_PWD2));
      oldpwd[2] = PasswordUtil.decrypt(upd.getItemValue(UserPwdDto.OLD_PWD3));
      for (int i = 0; i < oldpwd.length; i++) {
        if (pwd.equals(oldpwd[i])) return PRV_ERR;
      }
    }
    
    return null;
  }

  private final String PWDMSG = "PasswordChange";
  private final String PWDMOD = "PasswordMod";
  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface, com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage callControl(DetailItemsInterface dii,
      CommonMessage msg) throws Exception {
    _up.updatePassword(msg.getReq(PasswordChangeIni.MSG_UID), msg.getReq(PWD), PWD_FLG.NORMAL, session);
    msg.addMessage(resource.getString(PWDMSG));
    msg.setResMsg(PWDMOD, "1");
    return msg;
  }

  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new UserDto();
  }

}
