package com.hamaf.hikyaku.security.ui;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.text.MessageFormat;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.MultiCommonMessage;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.security.access.UserAccess.USER_TYPE;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.security.process.UserProcess;
import com.hamaf.hikyaku.ui.MultipartProcessBase;

/**
 * <p>^CgFUserMod4Excel is .</p>
 * <p>FUserListExcel쐬Unicode Text t@CɃ[U[XV</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2010/06/16</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class UserMod4Excel extends MultipartProcessBase {

	private UserProcess _up = null;

  /**
   *  Creates a new <code>UserMod4Excel.java</code> instance.
   * <br>Default Constructor
   * @param up
   */
  public UserMod4Excel(UserProcess up) {
    _up = up;
  }

  private final String NOT_FOUND = "UExlNotFound";
  private final String FORMAT_UNMATCH = "UExlFormatError";
  private final String MOD_END = "modend";
  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.ProcessBase#exec(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  public CommonMessage exec(CommonMessage msg) throws Exception {
    MultiCommonMessage mcmsg = (MultiCommonMessage)msg;
    String fl = mcmsg.getTempFileUrl();
    if (fl == null || fl.length() == 0) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.addMessage(super.resource.getString(NOT_FOUND));
      return msg;
    }
    File inf = new File(fl);
    if (!inf.exists()) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.addMessage(super.resource.getString(NOT_FOUND));
      return msg;
    }
    BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(inf), "UTF-16"));
    if (checkErrFile(br)) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.addMessage(super.resource.getString(FORMAT_UNMATCH));
      br.close();
      return msg;
    }
    msg.addMessage(update(br));
    br.close();
    msg.setResMsg(MOD_END, "1");
    return msg;
  }
  private final String TTL_VAL = "UExlTitle";
  private final String HED_VAL = "UExlHEader";
  private boolean checkErrFile(BufferedReader br) throws Exception {
    String buf = br.readLine();
    String[] dts = buf.split("\t");
    if (!resource.getString(TTL_VAL).equals(dts[0])) return true;
    buf = br.readLine();
    buf = br.readLine();
    String chk = buf.replaceAll("\t", ",").trim();
    if (!chk.equals(resource.getString(HED_VAL).trim())) return true;
    return false;
  }
  private final String MSGID = "UMod4Exl";
  private String update(BufferedReader br) throws Exception {
    String buf = br.readLine();
    int[] cnt = {0, 0, 0, 0};
    while (buf != null && buf.length() > 3) {
      String[] dts = buf.split("\t", -9);
      if (dts[1].length() > 0) cnt[0]++;
      String mod = dts[8].trim().toLowerCase();
      if (mod.length() > 0 && "amd".indexOf(mod) >= 0) {
        if ("a".equals(mod)) {
          UserDto ud = new UserDto();
          ud = setUSerData(ud, dts);
          ud.setItemValue(UserDto.USER_TYPE, USER_TYPE.GENERAL.getVal());
          ud.setItemValue(UserDto.INITIALIZE_FLG, com.hamaf.hikyaku.security.access.UserAccess.PWD_FLG.EMPTY.getVal());
          _up.insert(ud);
          String pw = dts[9].trim();
          if (pw != null && pw.length() > 0) {
          	_up.updatePassword(dts[1], pw, com.hamaf.hikyaku.security.access.UserAccess.PWD_FLG.INIT, session);
          }
          cnt[1]++;
        } else if ("m".equals(mod)) {
          UserDto ud = getUserInf(dts);
          _up.update(ud);
          cnt[2]++;
        } else {
          UserDto ud = getUserInf(dts);
          _up.delete(ud);
          cnt[3]++;
        }
      }
      buf = br.readLine();
    }
    Object[] prm = {String.valueOf(cnt[0]), String.valueOf(cnt[1]), String.valueOf(cnt[2]), String.valueOf(cnt[3])};
    return MessageFormat.format(resource.getString(MSGID), prm);
  }
  private UserDto setUSerData(UserDto ud, String[] dts) {
    ud.setItemValue(UserDto.USER_ID, dts[1]);
    ud.setItemValue(UserDto.NAME, dts[2]);
    ud.setItemValue(UserDto.ABBREV, dts[3]);
    ud.setItemValue(UserDto.ORGANIZATION, dts[4]);
    ud.setItemValue(UserDto.CONTACT, dts[5]);
    ud.setItemValue(UserDto.E_MAIL, dts[6]);
    ud.setItemValue(UserDto.ADMIN_FLG, dts[7]);

    Object udt = session.getAttribute(LogInCheck.USER_INF);
    if (udt != null && udt instanceof UserDto) {
      ud.setCommFld(COMMFLD.UUSR, ((UserDto)udt).getItemValue(UserDto.USER_ID));
    }
    return ud;
  }
  private UserDto getUserInf(String[] dts) throws Exception {
    UserDto ud = new UserDto();
    ud.setItemValue(UserDto.USER_ID, dts[1]);
    ud = _up.findByPrimarykey(ud, session);
    return setUSerData(ud, dts);
  }
}
