package com.hamaf.hikyaku.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

/**
 * <p>^CgFResourceUtil is .</p>
 * <p>F</p>
 * <br>
 * 
 * <p>쌠: Copyright (c) 2008/09/25</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class ResourceUtil {
  
  /**
   * Describe <code>getResource</code> method : \[Xoht@C̎擾.
   * <br>w肳ꂽname + "_ja_JP.properties" ̃\[Xoh擾
   * <br>w肳ꂽname + "_ja_JP.properties"File݂ꍇ͂A
   * <br>݂Ȃꍇ̓[_[𗘗pĎ擾܂B
   * 
   * @param name a <code>String</code> : \[X̖
   * @return a <code>ResourceBundle</code>
   */
  public static ResourceBundle getResource(String name) {
    if (name == null || name.length() == 0) {
      new MissingResourceException("File Name is null","","");
    }
    
    String fnm = name + "_ja_JP.properties";
    File fl = new File(fnm);
    if (fl.exists()) {
      try {
        return new PropertyResourceBundle(new FileInputStream(fl));        
      } catch (Exception xe) {
        new MissingResourceException(xe.getMessage(), "", fnm);
      }
    }
    
    return ResourceBundle.getBundle(name, Locale.JAPANESE);
  }
  
  /**
   * Describe <code>getProperty</code> method : vpeBt@C̎擾.
   * <br>w肳ꂽname + ".properties" ̃vpeB擾
   * <br>w肳ꂽname + ".properties"File݂ꍇ͂A
   * <br>݂Ȃꍇ̓[_[𗘗pĎ擾܂B
   * 
   * @param name a <code>String</code> : vpeBt@C̖
   * @return a <code>Properties</code> : 
   * @throws IOException
   */
  @SuppressWarnings("resource")
  public static Properties getProperty(String name) throws IOException {
    if (name == null || name.length() == 0) {
      throw new IOException("File Name is null!");
    }
    
    String fnm = name + ".properties";
    File fl = new File(fnm);
    InputStream ist = null;
    if (fl.exists()) {
      ist = new FileInputStream(fl);        
    }
    else {
      ClassLoader cl = Thread.currentThread().getContextClassLoader();
      ist = cl.getResourceAsStream(fnm);
    }
    if (ist == null) return null;
    
    Properties ret = new Properties();
    ret.load(ist);
    ist.close();
    return ret;
  }
  
  /**
   * Describe <code>getReader</code> method : t@C[_[̎擾.
   * <br>w肳ꂽnamew肳ꂽcharCodeœǂݍreader擾B
   * <br>w肳ꂽnameFile݂ꍇ͂A
   * <br>݂Ȃꍇ̓[_[𗘗pĎ擾܂B
   * 
   * @param name a <code>String</code> : t@C̖
   * @param chcd a <code>String</code> : GR[hinull̏ꍇ"Shift_JIS")
   * @return a <code>BufferedReader</code> : 
   * @throws IOException
   */
  @SuppressWarnings("resource")
  public static BufferedReader getReader(String name, String chcd) throws IOException {
    if (name == null || name.length() == 0) {
      throw new IOException("File Name is null!");
    }
    
    File fl = new File(name);
    InputStream ist = null;
    if (fl.exists()) {
      ist = new FileInputStream(fl);        
    }
    else {
      ClassLoader cl = Thread.currentThread().getContextClassLoader();
      ist = cl.getResourceAsStream(name);
    }
    if (chcd == null) chcd = "Shift_JIS";
    
    return new BufferedReader(new InputStreamReader(ist, chcd));
  }
  
  /**
   * Describe <code>splitCsv</code> method : ','؂CSVf[^zɂĖ߂.
   * <br>؂蕶","CSV`̃f[^zɂĖ߂B
   * 
   * @param org a <code>String</code> : csv`̃f[^
   * @return a <code>List&lt;String&gt;</code> : "," ŕ
   */
  public static List<String> splitCsv(String org) {
    return splitCsv(org, ',');
  }
  /**
   * Describe <code>splitCsv</code> method : CSVf[^zɂĖ߂.
   * <br>csv`̃f[^zɂĖ߂B
   * <ol>
   *   <li>"ň͂܂ꂽ͕ϊȂi؂蕶f[^Ƃj
   *   <li>"""ɕϊBAA""݂̂̃f[^͋󕶎Ƃ
   * </ol>
   * 
   * @param org a <code>String</code> : csv`̃f[^
   * @param dlm a <code>char</code> : ؂蕶
   * @return a <code>List&lt;String&gt;</code> : ؂蕶ŕ
   */
  public static List<String> splitCsv(String org, char dlm) {
    List<String> ldt = new ArrayList<String>();
    if (org.charAt(0) != '"') {
      ldt.add("");
    }
    ldt = cutWQuto(ldt, org);
    List<String> ret = new ArrayList<String>();
    for (int i = 0; i < ldt.size(); i++) {
      String vdt = (String)ldt.get(i);
      if ((i % 2) == 0) {
        if (vdt.trim().length() > 0) {
          ret.add(vdt);
        }
      }
      else {
        vdt = vdt.trim();
        if (vdt.length() > 0 && vdt.charAt(0) == dlm) {
          vdt = vdt.substring(1);
        }
        if ((i < ldt.size() - 1) &&
            vdt.length() > 0 && vdt.charAt(vdt.length() - 1) == dlm) {
          vdt = vdt.substring(0, vdt.length() - 1);
        }
        if (vdt.length() > 0) {
          String[] sp = vdt.split(Character.toString(dlm), -1);
          for (int j = 0; j < sp.length; j++) {
            if (sp[j].length() == 0 || "\"".equals(sp[j].trim())) {
              ret.add("");
            } else {
              ret.add(sp[j].trim());
            }
          }
        }
      }
    }
    return ret;
  }
  /**
   * Describe <code>cutWQuto</code> method : "ň͂܂ꂽ肾.
   * <br>"ň͂܂ꂽƂȊO̕ɕĖ߂B
   * 
   * @param spdt a <code>List&lt;String&gt;</code> : f[^ݒ肷z
   * @param org a <code>String</code> : 镶
   * @return a <code>List&lt;String&gt;</code> : f[^
   */
  private static List<String>  cutWQuto(List<String>  spdt, String org) {
    StringBuffer sb = new StringBuffer();
    boolean hit = false;
    for (int i = 0; i < org.length(); i++) {
      if (org.charAt(i) != '"') {
        sb.append(org.charAt(i));
      }
      else {
        if (i+1 < org.length() && org.charAt(i+1) == '"') {
          sb.append(org.charAt(i));
          i++;
        }
        else {
          if (hit) {
            spdt.add(sb.toString());
            return cutWQuto(spdt, org.substring(i + 1));
          }
          hit = true;
          if (sb.length() > 0) {
            if (sb.charAt(sb.length() - 1) != '"') {
              spdt.add(sb.toString());
              sb = new StringBuffer();
            } else {
              sb.setLength(sb.length() - 1);
              spdt.add(sb.toString());
              sb = new StringBuffer("\"");
            }
          }
        }
      }
    }
    if (sb.length() > 0) {
      spdt.add(sb.toString());
    }
    return spdt;
  }

}
