package com.hamaf.hikyaku.util;

import java.text.BreakIterator;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import com.hamaf.hikyaku.util.ComUtils;


/**
 * <p>^CgF֌W̃[eBeBNX.</p>
 * <p>Fȉ̋@\<code>static</code>ƂėLB</p>
 * <ol>
 *   <li>cnvJsp  : JSPpɃ^O̕ϊs
 *   <li>wordWrap: w肳ꂽoCgŃ[hbvs 
 *   <li>cnv2Han : Spp𔼊ppɕϊ
 *   <li>cnv2Zen : ppSppɕϊ
 *   <li>cnv2Asc : SpApp݂̂𔼊ppɕϊ
 *   <li>cnvKata2Hira : SpJ^Ji𕽉ɕϊ
 *   <li>cnvHira2Kata : SpJ^Jiɕϊ
 *   <li>kanaHan2Zen : pJ^JiSpJ^Jiɕϊ
 *   <li>kanaZen2Han : SpJ^Ji𔼊pJ^Jiɕϊ
 * </ol>
 * <p>쌠: Copyright (c) 2004</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class TextUtil {

  /**
   * Describe <code>cnvJsp</code> method : convert Tag characters.
   * <br>^OȂǂ̕ϊĖ߂B
   * @param dt an <code>Object</code> : ϊÕf[^
   * @param chrCd an <code>String</code> : R[h
   * @return a <code>String</code> : ϊ̕
   */
  public final static String cnvJsp(Object dt, String chrCd) {
    if (dt == null) {
      return "";
    }
    return ComUtils.win2sun(ComUtils.filter(dt.toString()), chrCd);
  }

  /**
   * Describe <code>wordWrap</code> method : word wrap.
   * <br>w肵oCgword wrap񃊃Xg߂B
   * 
   * @param dt a <code>String</code> : ΏۂƂ镶
   * @param len an <code>int</code> : s̃oCg
   * @return a <code>List&lt;String&gt;</code> : word wrap̕
   */
  public final static List<String> wordWrap(String dt, int len) {
    List<String> ret = new ArrayList<String>();
    if (dt == null || dt.length() == 0) {
      return ret;
    }
    if (len <1) {
      ret.add(dt);
      return ret;
    }
    
    BreakIterator bi = BreakIterator.getLineInstance(Locale.JAPAN);
    bi.setText(dt);
    int stp = bi.first();
    StringBuilder sb = new StringBuilder();
    int lnd = 0;
    for (int enp = bi.next(); enp != BreakIterator.DONE; stp = enp, enp = bi.next()) {
      String cdt = dt.substring(stp, enp);
      int dln = cdt.length();
      try {
        if (dln != cdt.getBytes("utf-8").length) {
          dln += dln;
        }
      } catch (Exception xe) {
        xe.printStackTrace();
      }
      
      if ((lnd + dln) > len) {
        if (sb.length() > 0) {
          ret.add(sb.toString());
        }
        sb.setLength(0);
        lnd = 0;
      }
      sb.append(cdt);
      lnd += dln;
    }
    if (sb.length() > 0) {
      ret.add(sb.toString());
    }
    return ret;
  }
  
  /**
   * Describe <code>cnvKata2Hira</code> method : convert Katakana to Hiragana.
   * <br>SpJ^Ji𕽉ɕϊB
   * <br>SpJ^JiȊO͂̂܂ܐݒ肷B
   * @param org a <code>String</code> : ϊΏۂ̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String cnvKata2Hira(String org) {
    if (org == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      char chr = org.charAt(i);
      if (chr >= 0x30A0 && chr <= 0x30FA) {
        chr -= 0x60;
      }
      sb.append(chr);
    }
    return sb.toString();
  }
  /**
   * Describe <code>cnvHira2Kata</code> method : convert Hiragana to Katakana.
   * <br>SpJ^JiɕϊB
   * <br>ȊO͂̂܂ܐݒ肷B
   * @param org a <code>String</code> : ϊΏۂ̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String cnvHira2Kata(String org) {
    if (org == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      char chr = org.charAt(i);
      if (chr >= 0x3040 && chr <= 0x309A) {
        chr += 0x60;
      }
      sb.append(chr);
    }
    return sb.toString();
  }    

  private static final String ZENKAKU = "@{|^bIHhfMijmnCDGFQO";
  private static final String HANKAKU = " +-*/=|!?\"#@$%&'`()[],.;:_<>^";
  /**
   * Describe <code>cnv2Han</code> method : convert 2byte to 1byte.
   * <br>Sp̉p(`-y-O-XL)𔼊p(A-Za-z0-9L)ɕϊ
   * <br>ϊłȂ͂̂܂ܐݒ肳܂B
   * @param org a <code>String</code> : ϊO̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String cnv2Han(String org) {
    if (org == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      char chr = org.charAt(i);
      int ix;
      if (chr >= 'O' && chr <= 'X') {
        chr += '0' - 'O';
      }
      else if (chr >= '`' && chr <= 'y') {
        chr += 'A' - '`';
      }
      else if (chr >= '' && chr <= '') {
        chr += 'a' - '';
      }
      else if ((ix = ZENKAKU.indexOf(chr)) > -1) {
        chr = HANKAKU.charAt(ix);
      }
      sb.append(chr);
    }
    return sb.toString();
  }

  /**
   * Describe <code>cnv2Zen</code> method : convert 1byte to 2byte.
   * <br>p(A-Za-z0-9L)Sp̉p(`-y-O-XL)ɕϊ
   * <br>ϊłȂ͂̂܂ܐݒ肳܂B
   * @param org a <code>String</code> : ϊO̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String cnv2Zen(String org) {
    if (org == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      char chr = org.charAt(i);
      int ix;
      if (chr >= '0' && chr <= '9') {
        chr += 'O' - '0';
      }
      else if (chr >= 'A' && chr <= 'Z') {
        chr += '`' - 'A';
      }
      else if (chr >= 'a' && chr <= 'z') {
        chr += '' - 'a';
      }
      else if ((ix = HANKAKU.indexOf(chr)) > -1) {
        chr = ZENKAKU.charAt(ix);
      }
      sb.append(chr);
    }
    return sb.toString();
  }
  
  /**
   * Describe <code>cnv2Asc</code> method : cnvort AscII char.
   * <br>Sp̉p(`-y-O-XL)𔼊pp(a-z0-9L)ɕϊB
   * <br>A-Z,a-z,0-9ȊO͖̕܂B
   * @param org a <code>String</code> : ϊO̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String cnv2Asc(String org) {
    if (org == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      int nv = Character.getNumericValue(org.charAt(i));
      if (nv > -1) {
        sb.append(Character.forDigit(nv, Character.MAX_RADIX));
      }
    }
    return sb.toString();
  }

  private static final String HANKANA =
    "¯֬ܦݰ";
  private static final String ZENKANA =
    "ACEGI@BDFHJLNPRTVXZ\^`cbegijklmnqtwz}~[AB";
  private static final String DAKUTEN = "";
  /**
   * Describe <code>kanaHan2Zen</code> method : convert Kana Hankaku to Zenkaku.
   * <br>pJiSpJiɕϊB
   * <br>_()͈ꕶɕϊ܂B
   * @param org a <code>String</code> : ϊΏۂ̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String kanaHan2Zen(String org) {
    if (org == null) {
      return "";
    }
    int idx;
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      char cr = org.charAt(i);
      if ((idx = HANKANA.indexOf(cr)) >= 0) {
        cr = ZENKANA.charAt(idx);
      }
      if ((idx = DAKUTEN.indexOf(cr)) >= 0) {
        cr = sb.charAt(sb.length() - 1);
        cr++;
        cr += idx;
        sb.setLength(sb.length() - 1);
      }
      sb.append(cr);
    }
    return sb.toString();
  }
  /**
   * Describe <code>kanaZen2Han</code> method : convert Kana Zenkaku to Hankaku.
   * <br>SpJ^Ji𔼊pJ^JiɕϊB
   * @param org a <code>String</code> : ϊΏۂ̕
   * @return a <code>String</code> : ϊ̕
   */
  public final static String kanaZen2Han(String org) {
    if (org == null) {
      return "";
    }
    int idx;
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < org.length(); i++) {
      char cr = org.charAt(i);
      char cr2 = cr;
      if ((idx = ZENKANA.indexOf(cr)) >= 0) {
        cr = HANKANA.charAt(idx);
      }
      else {
        cr2--;
        if ((idx = ZENKANA.indexOf(cr2)) >= 0) {
          sb.append(HANKANA.charAt(idx));
          cr = '';
        }
        else {
          cr2--;
          if ((idx = ZENKANA.indexOf(cr2)) >= 0) {
            sb.append(HANKANA.charAt(idx));
            cr = '';
          }
        }
      }
      sb.append(cr);
    }
    return sb.toString();
  }
  
  public static void main (String[] args) {
    System.out.println("-------- TextUtil Exec Sample -----------------------");
    System.out.println(" cnvJsp(\"<in& \">\") is " + cnvJsp("<in& \">", "Sift_JIS"));
//     System.out.println(" null is " + cnvJsp(null));
    System.out.println(" cnv2Han(\"AXdf`wOX\") is " +
                       cnv2Han("AXdf`wOX"));
    System.out.println(" cnv2Asc(\"AXdf`wOX\") is " +
                       cnv2Asc("AXdf`wOX"));
    System.out.println(" cnv2Zen(\"AXdf`w@%09\") is " +
                       cnv2Zen("AXdf`w@%09"));
    System.out.println(" cnvKata2Hira(\"AXA`w@%09\") is " +
                       cnvKata2Hira("AXA`w@%09"));
    System.out.println(" cnvHira2Kata(\"AXA`w@%09\") is " +
                       cnvHira2Kata("AXA`w@%09"));
    System.out.println(" kanaHan2Zen(\"߯ܤBެaZ9AKpbAB_\") is " +
                       kanaHan2Zen("߯ܤBެaZ9AKpbAB_"));
    System.out.println(" kanaZen2Han(\"߯ܤBެaZ9AKpbAB_\") is " +
                       kanaZen2Han("߯ܤBެaZ9AKpbAB_"));
    System.out.println("-------- TextUtil Sample End  -----------------------");
  } // end of main ()
  
}
