package com.hamaf.hikyakucms.sample.schedule.ui;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyakucms.sample.schedule.dto.ScheduleSearchDto;
import com.hamaf.hikyakucms.sample.schedule.dto.VScheduleDto;
import com.hamaf.hikyakucms.sample.schedule.process.HolidayProcess;
import com.hamaf.hikyakucms.sample.schedule.process.ScheduleProcess;
import com.hamaf.hikyakucms.sample.schedule.ui.mapping.VScheduleMap;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyaku.util.TextUtil;

/**
 * <p>^CgFPersonalMonthlyList is .</p>
 * <p>FԌlXPW[̕\</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/02/09</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class PersonalMonthlyList extends StandardListProcess {

	private ScheduleProcess _sp = null;
	private HolidayProcess _hp = null;
/**
	 * Creates a new <code>PersonalMonthlyList.java</code> instance.
	 * <br>Default Constructor
	 */
	public PersonalMonthlyList() {
		_jim = new VScheduleMap();
		_sp = new ScheduleProcess();
		_hp = new HolidayProcess();
	}

	private String sdt;
	private String edt;
	private String ssdt;
	private final String REQ_DATE = "sdate";
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
	 */
	@Override
	protected RetrieveKeysInterface setRetrieveVal(KeyList kl)
			throws IllegalRetrieveDataException {
		ScheduleSearchDto ssd = new ScheduleSearchDto();
		ssd.setRetrieveValue(ScheduleSearchDto.USER_ID, getUserID());
		String base = kl.getRetrieve(REQ_DATE);
		if (base == null || base.length() == 0) base = DateUtil.getDate();
		String offset = kl.getRetrieve("offset");
		if (offset == null || offset.length() == 0) offset = "0";
		base = DateUtil.addDateTime(base, Calendar.MONTH, Integer.parseInt(offset));
		ssdt = base;
		setStartEnd(base);
		StringBuilder sb = new StringBuilder();
		sb.append("'");
		sb.append(sdt);
		sb.append("' and '");
		sb.append(edt);
		sb.append("'");
		ssd.setRetrieveValue(ScheduleSearchDto.DAY, sb.toString());
		ssd.setCondition(ScheduleSearchDto.DAY, "BETWEEN");
		ssd.setOrderItem(ScheduleSearchDto.DAY, 1);
		ssd.setOrderItem(ScheduleSearchDto.TIME_FROM, 2);
		return ssd;
	}
	private void setStartEnd(String bdate) {
		if (bdate == null || bdate.length() == 0) bdate = DateUtil.getDate();
		Calendar scldr = DateUtil.getCalendar(bdate);
		scldr.set(Calendar.DATE, 1);
		Calendar ecldr = (Calendar)scldr.clone();
		int rg = 1 - scldr.get(Calendar.DAY_OF_WEEK) + 1;
		if (rg > 0) rg = rg - 7;
		scldr.add(Calendar.DATE, rg);
		ecldr.add(Calendar.MONTH, 1);
		ecldr.add(Calendar.DATE, -1);
		rg = (7 - ecldr.get(Calendar.DAY_OF_WEEK) + 1) % 7;
		ecldr.add(Calendar.DATE, rg);
		sdt = DateUtil.getFormatDate(scldr.getTime(), "yyyy/MM/dd");
		edt = DateUtil.getFormatDate(ecldr.getTime(), "yyyy/MM/dd");
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
	 */
	@Override
	protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
			throws Exception {
		return _sp.find((ScheduleSearchDto)rki);
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
	 */
	@Override
	protected DetailItemsInterface newDataObj() {
		return new VScheduleDto();
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
	 */
	@Override
	protected DetailItemsInterface callControl(DetailItemsInterface dii)
			throws Exception {
		return _sp.findviewbykey((VScheduleDto)dii, super.getUserID());
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
	 */
	@Override
	protected String[] checkDelete(DetailItemsInterface dii) {
		return null;
	}
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
	 */

	private final String MSG_TYPE = "datatype";
	private final String MSG_MMDD = "mmdd";
	@Override
	protected Map<String, String> editItems(DetailItemsInterface dii,
			Map<String, String> mp) throws Exception {
		String date = dii.getItemValue(VScheduleDto.DAY);
		mp.put(MSG_MMDD, date.substring(5));
		mp.put(MSG_TYPE, "9");
		return super.editItems(dii, mp);
	}

	private final String RES_UID = "uid";
  private final String RES_UNAME = "uname";
	private final String MSG_DD = "dd";
  private final String MSG_DYM = "dispyymm";
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.ProcessBase#finish(com.hamaf.hikyaku.dataobject.CommonMessage)
	 */
	@Override
	public CommonMessage finish(CommonMessage msg) {
		msg.setResMsg(REQ_DATE, ssdt);
    String[] dts = {ssdt};
    msg.setReq(REQ_DATE, dts);
		msg.setResMsg(RES_UID, getUserID());
    dts[0] = getUserID();
    msg.setReq(RES_UID, dts);
    UserDto ud = super.getUerDto();
    msg.setResMsg(RES_UNAME, ud.getItemValue(UserDto.NAME));
    dts[0] =  ud.getItemValue(UserDto.NAME);
    msg.setReq(RES_UNAME, dts);
		StringBuilder sb = new StringBuilder();
		sb.append(TextUtil.cnv2Zen(ssdt.substring(0, 4)));
		sb.append(" N ");
		sb.append(TextUtil.cnv2Zen(ssdt.substring(5, 7)));
		sb.append(" ");
		msg.setResMsg(MSG_DYM, sb.toString());
    dts[0] = sb.toString();
    msg.setReq(MSG_DYM, dts);

    List<Map<String, String>> odt = msg.getResList();
    List<Map<String, String>> ret = new ArrayList<Map<String,String>>();
    String bdt = sdt;
    int i = 0;
    while (bdt.compareTo(edt) <= 0) {
    	if ((i % 7) == 0) ret.add(createWeekMap(bdt));
    	i++;
    	ret.add(createDateMap(bdt));
    	for (Map<String, String> elm : odt) {
    		if (bdt.equals(elm.get("day"))) ret.add(elm);
    	}
    	bdt = DateUtil.addDateTime(bdt, Calendar.DATE, 1);
    }
    msg.setResList(ret);
		return super.finish(msg);
	}
	private final String MSG_BGCLR = "bgcolor";
	private Map<String, String> createWeekMap(String date) {
    Map<String, String> ret = new HashMap<String, String>();
    ret.put(POSNM, "");
 		ret.put("day", date);
 		ret.put(MSG_MMDD, date.substring(5));
 		ret.put(MSG_DD, date.substring(8));
		ret.put(MSG_TYPE, "1");
    return ret;
	}
	private Map<String, String> createDateMap(String date) {
    Map<String, String> ret = new HashMap<String, String>();
    ret.put(POSNM, "");
 		ret.put("day", date);
 		ret.put(MSG_MMDD, date.substring(5));
 		ret.put(MSG_DD, date.substring(8));
		ret.put(MSG_TYPE, "2");
		if (_hp.isHoliday(date, sysId)) {
			ret.put(MSG_BGCLR, "#ff66ff");
		} else {
			ret.put(MSG_BGCLR, "#ffffff");
		}
    return ret;
	}


}
