package com.hamaf.hikyakucms.sample.schedule.ui;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyakucms.sample.schedule.dto.ScheduleSearchDto;
import com.hamaf.hikyakucms.sample.schedule.dto.VScheduleDto;
import com.hamaf.hikyakucms.sample.schedule.process.HolidayProcess;
import com.hamaf.hikyakucms.sample.schedule.process.ScheduleProcess;
import com.hamaf.hikyakucms.sample.schedule.ui.mapping.VScheduleMap;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyaku.util.DateUtil;

/**
 * <p>^CgFPersonalScheduleList is .</p>
 * <p>FlXPW[̎擾</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/02/04</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class PersonalScheduleList extends StandardListProcess {

	private ScheduleProcess _sp = null;
	private HolidayProcess _hp = null;
	/**
	 * Creates a new <code>PersonalScheduleList.java</code> instance.
	 * <br>Default Constructor
	 */
	public PersonalScheduleList() {
		_jim = new VScheduleMap();
		_sp = new ScheduleProcess();
		_hp = new HolidayProcess();
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
	 */
	@Override
	protected DetailItemsInterface callControl(DetailItemsInterface dii)
			throws Exception {
		return _sp.findviewbykey((VScheduleDto)dii, super.getUserID());
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
	 */
	@Override
	protected String[] checkDelete(DetailItemsInterface dii) {
		return null;
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
	 */
	@Override
	protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
			throws Exception {
		return _sp.find((ScheduleSearchDto)rki);
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
	 */
	@Override
	protected DetailItemsInterface newDataObj() {
		return new VScheduleDto();
	}

	private String sdt;
	private String edt;
	private final String REQ_DATE = "sdate";
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
	 */
	@Override
	protected RetrieveKeysInterface setRetrieveVal(KeyList kl)
			throws IllegalRetrieveDataException {
		ScheduleSearchDto ssd = new ScheduleSearchDto();
		ssd.setRetrieveValue(ScheduleSearchDto.USER_ID, getUserID());
		String base = kl.getRetrieve(REQ_DATE);
		if (base == null || base.length() == 0) base = DateUtil.getDate();
		String offset = kl.getRetrieve("offset");
		if (offset == null || offset.length() == 0) offset = "0";
		sdt = DateUtil.addDateTime(base, Calendar.DATE, Integer.parseInt(offset) * 7);
		String sedt = DateUtil.addDateTime(sdt, Calendar.DATE, 6);
		edt = DateUtil.addDateTime(sdt, Calendar.DATE, 7);
		StringBuilder sb = new StringBuilder();
		sb.append("'");
		sb.append(sdt);
		sb.append("' and '");
		sb.append(sedt);
		sb.append("'");
		ssd.setRetrieveValue(ScheduleSearchDto.DAY, sb.toString());
		ssd.setCondition(ScheduleSearchDto.DAY, "BETWEEN");
		ssd.setOrderItem(ScheduleSearchDto.DAY, 1);
		ssd.setOrderItem(ScheduleSearchDto.TIME_FROM, 2);
		return ssd;
	}

	private final String MSG_YOBI = "yobi";
	private final String MSG_HOLIDAY = "holiday";
	private final String MSG_TOP = "top";
	private final String MSG_LEFT = "left";
	private final String MSG_HEIGHT = "height";
	private final String TOP_TIME = "08:00";
	private final String FOOT_TIME = "19:30";
	private final int TOP_MIN = 60;
	private final int TOP_MAX = (11 * 60) + 30 + 60;
	private final int DEF_HEIGHT = 60;
	private final int LEFT_BASE = 10;
	private final int LEFT_STEP = 150;
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
	 */
	@Override
	protected Map<String, String> editItems(DetailItemsInterface dii,
			Map<String, String> mp) throws Exception {
		String stm = dii.getItemValue(VScheduleDto.TIME_FROM);
		int itop = TOP_MIN;
		if (stm != null && stm.length() > 0) {
			if (stm.compareTo(TOP_TIME) > 0) {
				if (stm.compareTo(FOOT_TIME) < 0) {
					itop += rangeMinute(TOP_TIME, stm);
				} else {
					itop = TOP_MAX;
				}
			}
		}
		mp.put(MSG_TOP, String.valueOf(itop));
		int ihg = DEF_HEIGHT;
		if (stm != null && stm.length() > 0) {
			String etm = dii.getItemValue(VScheduleDto.TIME_TO);
			if (etm != null && etm.length() > 0) {
				ihg = rangeMinute(stm, etm);
			}
		}
		mp.put(MSG_HEIGHT, String.valueOf(ihg));
		String dt = dii.getItemValue(VScheduleDto.DAY);
		long sd = DateUtil.getCalendar(sdt).getTimeInMillis();
		long ed = DateUtil.getCalendar(dt).getTimeInMillis();
		long rd = (ed - sd) / (24 * 60 * 60 * 1000);
		rd *= LEFT_STEP;
		rd += LEFT_BASE;
		mp.put(MSG_LEFT, String.valueOf(rd));
		mp.put(MSG_YOBI, DateUtil.getYoubi(dt));
		if (_hp.isHoliday(dt, sysId)) {
			mp.put(MSG_HOLIDAY, "1");
		} else {
			mp.put(MSG_HOLIDAY, "0");
		}
		return super.editItems(dii, mp);
	}
	private int rangeMinute(String from, String to) {
		int fh = Integer.parseInt(from.substring(0, 2));
		int th = Integer.parseInt(to.substring(0, 2));
		int fm = Integer.parseInt(from.substring(3));
		int tm = Integer.parseInt(to.substring(3));
		return ((th - fh) * 60) + (tm - fm);
	}

  private final String RES_UID = "uid";
  private final String RES_UNAME = "uname";
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.ProcessBase#finish(com.hamaf.hikyaku.dataobject.CommonMessage)
	 */
	@Override
	public CommonMessage finish(CommonMessage msg) {
		msg.setErrorStatus(STATUS.NORMAL);
		msg.setResMsg(REQ_DATE, sdt);
		msg.setResMsg(RES_UID, getUserID());
		msg.setResMsg(MSG_YOBI, DateUtil.getYoubi(sdt));
    UserDto ud = super.getUerDto();
    msg.setResMsg(RES_UNAME, ud.getItemValue(UserDto.NAME));

    List<Map<String, String>> mdt = msg.getResList();
		List<List<String>> adt = getAllDate();
		msg.setWorkBuff(adt);
		int i = 0;
		String mdate = edt;
		if (mdt.size() > 0) mdate = mdt.get(i).get("day");
		for (List<String> elm : adt) {
			if (elm.get(0).compareTo(mdate) < 0) {
				mdt.add(i, createNullMap(elm.get(0)));
				i++;
			} else {
				while (elm.get(0).compareTo(mdate) >= 0) {
					i++;
					if (i < mdt.size()) {
						mdate = mdt.get(i).get("day");
					} else {
						mdate = edt;
					}
				}
			}
		}
		msg.setResList(mdt);
		return super.finish(msg);
	}

	private final String BACKIMG_NORMAL = "plugins/schedule/jsp/contents/scheduleback.jpg";
	private final String BACKIMG_HOLIDAY = "plugins/schedule/jsp/contents/schedulebackHld.jpg";
	private List<List<String>> getAllDate() {
		List<List<String>> ret = new ArrayList<List<String>>();
		int left = LEFT_BASE;
		String wdt = sdt;
		while (wdt.compareTo(edt) < 0) {
			List<String> elm = new ArrayList<String>();
			elm.add(wdt);
			elm.add(DateUtil.getYoubi(wdt));
			elm.add(String.valueOf(left));
			left += LEFT_STEP;
			if (_hp.isHoliday(wdt, sysId)) {
				elm.add(BACKIMG_HOLIDAY);
			} else {
				elm.add(BACKIMG_NORMAL);
			}
			ret.add(elm);
			wdt = DateUtil.addDateTime(wdt, Calendar.DATE, 1);
		}
		return ret;
	}
	private Map<String, String> createNullMap(String date) {
    Map<String, String> ret = new HashMap<String, String>();
    ret.put(POSNM, "");
    for (String kv : _jim) {
    	if (kv.equals("day")) {
    		ret.put(kv, date);
    	}else {
    		ret.put(kv, "");
    	}
    }
		ret.put(MSG_YOBI, DateUtil.getYoubi(date));
		if (_hp.isHoliday(date, sysId)) {
			ret.put(MSG_HOLIDAY, "1");
		} else {
			ret.put(MSG_HOLIDAY, "0");
		}
    return ret;
	}

}
