package com.hamaf.hikyakucms.sample.schedule.ui;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import com.hamaf.hikyakucms.sample.schedule.dto.ScheduleDto;
import com.hamaf.hikyakucms.sample.schedule.process.ScheduleProcess;
import com.hamaf.hikyakucms.sample.schedule.ui.mapping.ScheduleMap;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.extension.MasterXmlControl;
import com.hamaf.hikyaku.extension.MasterXmlFactory;
import com.hamaf.hikyaku.ui.StandardCheckAndUpdate;
import com.hamaf.hikyaku.util.DateUtil;

/**
 * <p>^CgFScheduleAdd is .</p>
 * <p>FXPW[o^ʏ</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2010/08/10</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class ScheduleAdd extends StandardCheckAndUpdate {

  private ScheduleProcess _sp = null;
  private final String CHK_FLD = "ScheduleAdd";
  private final String PLGID = "schedule";

  /**
   *  Creates a new <code>ScheduleAdd.java</code> instance.
   * <br>Default Constructor
   */
  public ScheduleAdd() {
    _sp = new ScheduleProcess();
    _jim = new ScheduleMap();
  }

  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#setFolder()
   */
  @Override
  protected String setFolder() {
    return CHK_FLD;
  }

  private final String ERR_ET = "err_time";
  private final String ERR_ND = "err_not_data";
  private final String REP_END = "repEndDay";
  private final String REP_TYP = "repType";
  private final String REP_TYP_DAY = "daily";
  private final String REP_TYP_WEEK = "weekly";
  private final String REP_TYP_MDAY = "monthlyDay";
  private final String REP_TYP_MWEEK = "monthlyWeek";
  private final String REP_WEEK = "repWeek";
  private final String REP_WEEK_ER = "repWeekErr";
  private final String REP_DAY1 = "repMDay1";
  private final String REP_DAY2 = "repMDay2";
  private final String REP_DAY3 = "repMDay3";
  private final String REP_WEEKN = "repWeekNumber";
  private final String REP_MWEEK = "repMonthWeek";
  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#checkRelation(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage checkRelation(CommonMessage msg) throws Exception {
    String tto = msg.getReqStr("time_to");
    if (tto.length() == 0) return msg;
    if (tto.compareTo(msg.getReqStr("time_from")) < 0) msg = setError(msg, ERR_ET, "time_to");
    String repEnd = msg.getReqStr(REP_END);
    if (repEnd.length() == 0) return msg;

    String repTyp = msg.getReqStr(REP_TYP);
    if (repTyp.equals(REP_TYP_WEEK)) {
    	if (msg.getReqStr(REP_WEEK).length() == 0) msg = setError(msg, ERR_ND, REP_WEEK_ER);
    } else if (repTyp.equals(REP_TYP_MDAY)) {
			if (msg.getReqStr(REP_DAY1).length() == 0 &&
					msg.getReqStr(REP_DAY2).length() == 0 &&
					msg.getReqStr(REP_DAY3).length() == 0) msg = setError(msg, ERR_ND, REP_DAY1);
		} else if (repTyp.equals(REP_TYP_MWEEK)) {
    	if (msg.getReqStr(REP_WEEKN).length() == 0) msg = setError(msg, ERR_ND, REP_WEEKN);
		}
    return msg;
  }
  private final String TAG = "message";
  private final String VAL_TAG = "messageName";
  private CommonMessage setError(CommonMessage msg, String ecd, String fld) {
    msg.setErrorStatus(STATUS.WARNING);
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(PLGID);
    msg.setResMsg(fld, mxc.getTagValue(TAG, ecd, VAL_TAG));
  	return msg;
  }

  private final String OKFLD = "scheduleAdd";
  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface, com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage callControl(DetailItemsInterface dii,
      CommonMessage msg) throws Exception {
    String repEnd = msg.getReqStr(REP_END);
    if (repEnd.length() == 0) {
    	_sp.register((ScheduleDto)dii);
    } else {
      String repTyp = msg.getReqStr(REP_TYP);
      if (repTyp.equals(REP_TYP_DAY)) {
      	addDaily((ScheduleDto)dii, msg);
      } else if (repTyp.equals(REP_TYP_WEEK)) {
      	addWeek((ScheduleDto)dii, msg);
      } else if (repTyp.equals(REP_TYP_MDAY)) {
				addMonthDay((ScheduleDto)dii, msg);
			} else if (repTyp.equals(REP_TYP_MWEEK)) {
				addMunthWeek((ScheduleDto)dii, msg);
			}
    }
    msg.setResMsg(OKFLD, "1");
    return msg;
  }

  private void addDaily(ScheduleDto scd, CommonMessage msg) throws Exception {
  	String bdt = scd.getItemValue(ScheduleDto.DAY);
  	String edt = msg.getReqStr(REP_END);
  	while (bdt.compareTo(edt) <= 0) {
  		scd.setItemValue(ScheduleDto.DAY, bdt);
  		_sp.register(scd);
    	bdt = DateUtil.addDateTime(bdt, Calendar.DATE, 1);
  	}
  }
  private void addWeek(ScheduleDto scd, CommonMessage msg) throws Exception {
  	String sdt = scd.getItemValue(ScheduleDto.DAY);
  	String edt = msg.getReqStr(REP_END);
  	String[] yobis = msg.getReqValus(REP_WEEK);
  	for (int i = 0; i < yobis.length; i++) {
			String yobi = yobis[i];
			String bdt = sdt;
			while (!yobi.equals(DateUtil.getYoubi(bdt)) && bdt.compareTo(edt) <= 0) {
	    	bdt = DateUtil.addDateTime(bdt, Calendar.DATE, 1);
			}
	  	while (bdt.compareTo(edt) <= 0) {
	  		scd.setItemValue(ScheduleDto.DAY, bdt);
	  		_sp.register(scd);
	    	bdt = DateUtil.addDateTime(bdt, Calendar.DATE, 7);
	  	}
		}
  }
  private void addMonthDay(ScheduleDto scd, CommonMessage msg) throws Exception {
  	String sdt = scd.getItemValue(ScheduleDto.DAY);
  	String edt = msg.getReqStr(REP_END);
  	List<String> dates = new ArrayList<String>();
  	String rdt = msg.getReqStr(REP_DAY1);
  	if (rdt.length() > 0) dates.add(rdt);
  	rdt = msg.getReqStr(REP_DAY2);
  	if (rdt.length() > 0) dates.add(rdt);
  	rdt = msg.getReqStr(REP_DAY3);
  	if (rdt.length() > 0) dates.add(rdt);
  	for (String date : dates) {
			String bdt = cnvEndDate(sdt, date, false);
	  	while (bdt.compareTo(edt) <= 0) {
	  		if (bdt.compareTo(sdt) >= 0) {
	  			scd.setItemValue(ScheduleDto.DAY, bdt);
	  			_sp.register(scd);
	  		}
	    	bdt = cnvEndDate(bdt, date, true);
	  	}
		}
  }
  private String cnvEndDate(String ymd, String dd, boolean month) {
  	if (!"31".equals(dd)) {
  		if (month) {
  			return DateUtil.addDateTime(ymd, Calendar.MONTH, 1);
  		}
  		return DateUtil.setDateTime(ymd, Calendar.DATE, Integer.parseInt(dd));
  	}
  	int mm = 1;
  	if (month) mm = 2;
  	String dt = DateUtil.setDateTime(ymd, Calendar.DATE, 1);
  	dt = DateUtil.addDateTime(dt, Calendar.MONTH, mm);
  	return DateUtil.addDateTime(dt, Calendar.DATE, -1);
  }
  private void addMunthWeek(ScheduleDto scd, CommonMessage msg) throws Exception {
  	String sdt = scd.getItemValue(ScheduleDto.DAY);
  	String edt = msg.getReqStr(REP_END);
  	String yobi = msg.getReqStr(REP_MWEEK);
  	int week = Integer.parseInt(msg.getReqStr(REP_WEEKN));
  	week = (week - 1) * 7;
  	String bdt = DateUtil.setDateTime(sdt, Calendar.DATE, 1);
  	while (bdt.compareTo(edt) <= 0) {
    	while (!yobi.equals(DateUtil.getYoubi(bdt)) && bdt.compareTo(edt) <= 0) {
      	bdt = DateUtil.addDateTime(bdt, Calendar.DATE, 1);
    	}
    	String svym = bdt.substring(0,7);
			bdt = DateUtil.addDateTime(bdt, Calendar.DATE, week);
  		if (bdt.compareTo(sdt) >= 0 && bdt.compareTo(edt) <= 0 && bdt.startsWith(svym)) {
  			scd.setItemValue(ScheduleDto.DAY, bdt);
    		_sp.register(scd);
  		}
    	bdt = DateUtil.setDateTime(bdt, Calendar.DATE, 1);
			if (bdt.startsWith(svym)) bdt = DateUtil.addDateTime(bdt, Calendar.MONTH, 1);
  	}
  }

  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new ScheduleDto();
  }

}
