package com.hamaf.hikyakucms.sample.sns.managerUi;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationDto;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.ConversationProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.ConversionMap;

/**
 * <p>^CgFConversationList is .</p>
 * <p>FǗҗp⍇ꗗ/p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/04/08</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class ConversationList extends StandardListProcess {

  private ConversationProcess _cp = null;
  private boolean _ret = false;

  /**
   * Creates a new <code>ConversationList.java</code> instance.
   * <br>Default Constructor
   */
  public ConversationList() {
    _jim = new ConversionMap();
    _cp = new ConversationProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
      throws IllegalRetrieveDataException {
    ConversationSearchDto csd = new ConversationSearchDto();
    if (keyl.getRetrieve("ret").length() == 0) return csd;

    _ret = true;
    String fdate = keyl.getRetrieve("sfinishDay");
    if (fdate.length() > 0) {
      csd.setRetrieveValue(ConversationSearchDto.FINISH_DATE, fdate);
      csd.setCondition(ConversationSearchDto.FINISH_DATE, "<=");
    }
    String edate = keyl.getRetrieve("sentryDay");
    if (edate.length() > 0) {
      csd.setRetrieveValue(ConversationSearchDto.ENTRY_DATE, edate);
      csd.setCondition(ConversationSearchDto.ENTRY_DATE, "<=");
    }
    String ttl = keyl.getRetrieve("stitle");
    if (ttl.length() > 0) {
      csd.setRetrieveValue(ConversationSearchDto.TITLE, "%" + ttl + "%");
      csd.setCondition(ConversationSearchDto.TITLE, "LIKE");
    }
    String dtl = keyl.getRetrieve("sdetail");
    if (dtl.length() > 0) {
      csd.setRetrieveValue(ConversationSearchDto.DETAIL, "%" + dtl + "%");
      csd.setCondition(ConversationSearchDto.DETAIL, "LIKE");
    }
    csd.setRetrieveValue(ConversationSearchDto.ENTRY_USER, keyl.getRetrieve("suser"));
    csd.setRetrieveValue(ConversationSearchDto.LEVEL, "1");
    csd.setSysId(sysId);
    try {
      csd = (ConversationSearchDto)_cp.find(csd);
    }
    catch (Exception ex) {
      throw new IllegalRetrieveDataException(ex.toString(), new HashMap<String, String>());
    }
    StringBuilder sb = new StringBuilder();
    for (List<String> keys : csd) {
      sb.append("'");
      sb.append(keys.get(0));
      sb.append("',");
    }
    if (sb.length() == 0) return csd;
    ConversationSearchDto csd2 = new ConversationSearchDto();
    sb.setLength(sb.length() - 1);
    csd2.setRetrieveValue(ConversationSearchDto.ORIGINAL_SEQ, sb.toString());
    csd2.setCondition(ConversationSearchDto.ORIGINAL_SEQ, "IN");
    csd2.setOrderItem(ConversationSearchDto.SEQUENCE, 1);
    return csd2;
  }

  private final String MSG_MGN = "leftmargin";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii,
      Map<String, String> mp) throws Exception {
    String slv = dii.getItemValue(ConversationDto.LEVEL);
    int ilv = 0;
    if (slv != null && slv.length() > 0) {
      ilv = Integer.parseInt(slv);
      ilv --;
    }
    mp.put(MSG_MGN, String.valueOf(ilv * 15));
    return super.editItems(dii, mp);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    if (_ret) rki = _cp.find((ConversationSearchDto)rki);
    return rki;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new ConversationDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _cp.findByPrimaryKey((ConversationDto)dii);
  }

  private final String MSG_DEL = "*** 폜܂ ***";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    if (delCheck(dii)) {
      String[] ddt = {"detail", MSG_DEL};
      return ddt;
    }
    return null;
  }

}
