package com.hamaf.hikyakucms.sample.sns.managerUi;

import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyakucms.sample.sns.dto.NoticeDto;
import com.hamaf.hikyakucms.sample.sns.dto.NoticeSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.NoticeProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.NoticeMap;

/**
 * <p>^CgFNoticeList is .</p>
 * <p>FǗҗpm点ꗗ\</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/04/04</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class NoticeList extends StandardListProcess {

  private NoticeProcess _np = null;
  private SecurityCheckUtils _scu = null;
  private boolean _ret = false;

  /**
   * Creates a new <code>NoticeList.java</code> instance.
   * <br>Default Constructor
   */
  public NoticeList() {
    _jim = new NoticeMap();
    _np = new NoticeProcess();
    _scu = new SecurityCheckUtils();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
      throws IllegalRetrieveDataException {
    NoticeSearchDto nsd = new NoticeSearchDto();
    if (keyl.getRetrieve("ret").length() == 0) return nsd;

    _ret = true;
    nsd.setRetrieveValue(NoticeSearchDto.USER_ID, keyl.getRetrieve("sreciveUser"));
    nsd.setRetrieveValue(NoticeSearchDto.ENTRY_USER, keyl.getRetrieve("sentryUser"));
    String rdt = keyl.getRetrieve("sentryDay");
    if (rdt != null && rdt.length() > 0) {
      nsd.setRetrieveValue(NoticeSearchDto.ENTRY_DAY, rdt);
      nsd.setCondition(NoticeSearchDto.ENTRY_DAY, "<=");
    }
    rdt = keyl.getRetrieve("saffirmationDay");
    if ("1".equals(keyl.getRetrieve("snocheck"))) {
      nsd.setRetrieveValue(NoticeSearchDto.AFFIRMATION_DAY, "0000/00/00");
    } else {
      if (rdt != null && rdt.length() > 0) {
        StringBuilder sb2 = new StringBuilder();
        sb2.append("'0000/01/01' AND '");
        sb2.append(rdt);
        sb2.append("'");
        nsd.setRetrieveValue(NoticeSearchDto.AFFIRMATION_DAY, sb2.toString());
        nsd.setCondition(NoticeSearchDto.AFFIRMATION_DAY, "BETWEEN");
      }
    }
    rdt = keyl.getRetrieve("scontents");
    if (rdt != null && rdt.length() > 0) {
      nsd.setRetrieveValue(NoticeSearchDto.CONTENTS, "%" + rdt + "%");
      nsd.setCondition(NoticeSearchDto.CONTENTS, "LIKE");
    }

    nsd.setOrderItem(NoticeSearchDto.ENTRY_DAY, -1);
    nsd.setOrderItem(NoticeSearchDto.AFFIRMATION_DAY, 2);
    nsd.setOrderItem(NoticeSearchDto.USER_ID, 3);

    return nsd;
  }

  private final String MSG_UNM = "userName";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii,
      Map<String, String> mp) throws Exception {
    UserDto ud = _scu.getUserInf(dii.getItemValue(NoticeDto.USER_ID), getUserID());
    String unm = ud.getItemValue(UserDto.ABBREV);
    if (unm == null || unm.length() == 0) unm = ud.getItemValue(UserDto.NAME);
    mp.put(MSG_UNM, unm);
    if (dii.getItemValue(NoticeDto.AFFIRMATION_DAY).startsWith("0000")) {
      mp.put("affirmationDay", " ");
    }
    return mp;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki) throws Exception {
    if (_ret) rki = _np.find((NoticeSearchDto)rki);
    return rki;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new NoticeDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii) throws Exception {
    return _np.findByPrimaryKey((NoticeDto)dii);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    if (delCheck(dii)) {
      String[] ret = {"contents", "### 폜f[^ł"};
      return ret;
    }
    return null;
  }

}
