package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access.HATSUBAN2_KBN;
import com.hamaf.hikyakucms.sample.sns.dto.VCommunicationDto;
import com.hamaf.hikyakucms.sample.sns.dto.VCommunicationSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.CommunicationProcess;
import com.hamaf.hikyakucms.sample.sns.process.FootprintsProcess;
import com.hamaf.hikyakucms.sample.sns.process.RestrictionProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.VCommunicationMap;

/**
 * <p>^CgFCommunicationList is .</p>
 * <p>FAꗗ\</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/05</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class CommunicationList extends StandardListProcess {

  private CommunicationProcess _cp = null;
  private FootprintsProcess _fp = null;
  private RestrictionProcess _rp = null;

  /**
   * Creates a new <code>CommunicationList.java</code> instance.
   * <br>Default Constructor
   */
  public CommunicationList() {
    _jim = new VCommunicationMap();
    _cp = new CommunicationProcess();
    _fp = new FootprintsProcess();
    _rp = new RestrictionProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl) throws IllegalRetrieveDataException {
    VCommunicationSearchDto vcsd = new VCommunicationSearchDto();
    vcsd.setRetrieveValue(VCommunicationSearchDto.GROUP_ID, _rp.getGropuCondition(getUserID(), false));
    vcsd.setCondition(VCommunicationSearchDto.GROUP_ID, "IN");
    String today = DateUtil.getDate();
    vcsd.setRetrieveValue(VCommunicationSearchDto.DAY_FROM, today);
    vcsd.setCondition(VCommunicationSearchDto.DAY_FROM, "<=");
    vcsd.setRetrieveValue(VCommunicationSearchDto.DAY_TO, today);
    vcsd.setCondition(VCommunicationSearchDto.DAY_TO, ">=");

    vcsd.setRetrieveValue(VCommunicationSearchDto.GENRE, keyl.getRetrieve("sgener"));
    String entusr = keyl.getRetrieve("sentryName");
    if (entusr != null && entusr.length() > 0) {
      vcsd.setRetrieveValue(VCommunicationSearchDto.ENTRY_NAME, entusr);
      vcsd.setCondition(VCommunicationSearchDto.ENTRY_NAME, "LIKE");
    }

    String sval = keyl.getRetrieve("key");
    if (sval != null && sval.length() > 0) {
      sval = "%" + sval + "%";
      vcsd.setRetrieveValue(VCommunicationSearchDto.RETVAL, sval);
      vcsd.setCondition(VCommunicationSearchDto.RETVAL, "LIKE");
    }
    vcsd.setOrderItem(VCommunicationSearchDto.SEQNO, -1);
    return vcsd;
  }

  private final String MSG_FP = "footprints";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii, Map<String, String> mp) throws Exception {
    String sfp = _keys.getRetrieve("sfoot");
    if ("1".equals(sfp)) {
      mp.put(MSG_FP, " ");
      return mp;
    }
    if ("1".equals(_fp.isFootprints(HATSUBAN2_KBN.COMMUNICATION, dii.getItemValue(VCommunicationDto.SEQNO), sysId, getUserID()))) {
      mp.put(MSG_FP, "mFς");
    } else {
      mp.put(MSG_FP, " ");
    }
    return super.editItems(dii, mp);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _cp.find((VCommunicationSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#selectKeys(java.util.List)
   */
  @Override
  protected List<List<String>> selectKeys(List<List<String>> keys) {
    try {
      String sfp = _keys.getRetrieve("sfoot");
      if ("1".equals(sfp)) {
        List<List<String>> nkey = new ArrayList<List<String>>();
        for (List<String> keylst : keys) {
          if ("0".equals(_fp.isFootprints(HATSUBAN2_KBN.COMMUNICATION, keylst.get(0), sysId, getUserID()))) {
            nkey.add(keylst);
          }
        }
        return nkey;
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    return super.selectKeys(keys);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new VCommunicationDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _cp.getDetail((VCommunicationDto)dii);
  }

  private final String MSG_DEL = "*** 폜Ă܂ ***";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    if (delCheck(dii)) {
      String[] ddt = {"contents", MSG_DEL};
      return ddt;
    }
    return null;
  }

}
