package com.hamaf.hikyakucms.sample.sns.ui;

import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.ui.StandardCheckAndUpdate;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyakucms.sample.sns.dto.CommunicationDto;
import com.hamaf.hikyakucms.sample.sns.process.CommunicationProcess;
import com.hamaf.hikyakucms.sample.sns.process.RestrictionProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.CommunicationMap;

/**
 * <p>^CgFCommunicationUpdate is .</p>
 * <p>FA̓o^EXV</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/05</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class CommunicationUpdate extends StandardCheckAndUpdate {

  private final String CHK_FLDR = "CommunicationUpdate";
  private CommunicationProcess _cp = null;
  private RestrictionProcess _rp = null;

  /**
   * Creates a new <code>CommunicationUpdate.java</code> instance.
   * <br>Default Constructor
   */
  public CommunicationUpdate() {
    _jim = new CommunicationMap();
    _cp = new CommunicationProcess();
    _rp = new RestrictionProcess();
    _cfld = CHK_FLDR;
  }

  private final String ERR_DAY = "JnI߂Ă܂B";
  private final String ERR_TDY = "I𓖓ߋtɂȂĂ܂B";
  private final String ERR_GRP = "ΏۃO[v܂͑S[UIĂ܂B";
  private final String ERR_MGR = "Ǘ҂w肳ꂽO[vɑĂ܂B";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#checkRelation(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage checkRelation(CommonMessage msg) throws Exception {
    String fdy = msg.getReqStr("dayFrom");
    String tdy = msg.getReqStr("dayTo");
    if (fdy.length() > 0 && tdy.length() > 0) {
      if (fdy.compareTo(tdy) > 0) {
        msg.setErrorStatus(STATUS.WARNING);
        msg.setResMsg("dayFrom", ERR_DAY);
      }
    }
    if (tdy.length() > 0 && tdy.compareTo(DateUtil.getDate()) < 0) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.setResMsg("dayTo", ERR_TDY);
    }
    String alf = msg.getReqStr("allFlg");
    if ("1".equals(alf)) return msg;

    String grp = msg.getReqStr(CommunicationDetail.MSG_GID);
    if (grp.length() == 0) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.setResMsg(CommunicationDetail.MSG_GID, ERR_GRP);
    }
    String mgr = msg.getReqStr("manager");
    if (!_rp.checkIngroup(mgr, grp)) {
      msg.setErrorStatus(STATUS.WARNING);
      msg.setResMsg("managerName", ERR_MGR);
    }
    return msg;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new CommunicationDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface, com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage callControl(DetailItemsInterface dii, CommonMessage msg) throws Exception {
    CommunicationDto cd = (CommunicationDto)dii;
    if (!msg.getReqStr("allFlg").equals("1")) {
      cd.setItemValue(CommunicationDto.ALL_FLG, "0");
      String[] rest = msg.getReqStr(CommunicationDetail.MSG_GID).split(",");
      for (int i = 0; i < rest.length; i++) {
        cd.addRestriction(rest[i]);
      }
    }
    String seq = dii.getItemValue(CommunicationDto.SEQNO);
    if (seq == null || seq.length() == 0) {
      String nno = _cp.regist(cd);
      msg.setResMsg("seqNo", nno);
    } else {
      _cp.update(cd);
    }
    return msg;
  }

}
