package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyakucms.sample.sns.dto.EnqueteDto;
import com.hamaf.hikyakucms.sample.sns.dto.VEnqueteSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.AnswerProcess;
import com.hamaf.hikyakucms.sample.sns.process.EnqueteProcess;
import com.hamaf.hikyakucms.sample.sns.process.RestrictionProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.EnqueteMap;

/**
 * <p>^CgFEnqueteList is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/25</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class EnqueteList extends StandardListProcess {

  private EnqueteProcess _ep = null;
  private RestrictionProcess _rp = null;
  private AnswerProcess _ap = null;

  /**
   * Creates a new <code>EnqueteList.java</code> instance.
   * <br>Default Constructor
   */
  public EnqueteList() {
    _jim = new EnqueteMap();
    _ep = new EnqueteProcess();
    _rp = new RestrictionProcess();
    _ap = new AnswerProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
      throws IllegalRetrieveDataException {
    VEnqueteSearchDto vesd = new VEnqueteSearchDto();
    vesd.setRetrieveValue(VEnqueteSearchDto.GROUP_ID, _rp.getGropuCondition(getUserID(), true));
    vesd.setCondition(VEnqueteSearchDto.GROUP_ID, "IN");
    vesd.setRetrieveValue(VEnqueteSearchDto.FINISH_DAY, "NULL");
    vesd.setCondition(VEnqueteSearchDto.FINISH_DAY, "IS");

    String pa = keyl.getRetrieve("sPossibleAnswers");
    if ("1".equals(pa)) {
      vesd.setRetrieveValue(VEnqueteSearchDto.DAY_TO, DateUtil.getDate());
      vesd.setCondition(VEnqueteSearchDto.DAY_TO, "=>");
    }
    String entusr = keyl.getRetrieve("sentryName");
    if (entusr != null && entusr.length() > 0) {
      vesd.setRetrieveValue(VEnqueteSearchDto.ENTRY_NAME, entusr);
      vesd.setCondition(VEnqueteSearchDto.ENTRY_NAME, "LIKE");
    }
    return vesd;
  }

  private final String MSG_ANS = "answer";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii, Map<String, String> mp) throws Exception {
    String sfp = _keys.getRetrieve("sans");
    if ("1".equals(sfp)) {
      mp.put(MSG_ANS, " ");
      return mp;
    }
    if (_ap.isAnswer(dii.getItemValue(EnqueteDto.SEQNO), getUserID(), sysId)) {
      mp.put(MSG_ANS, "񓚍ς");
    } else {
      mp.put(MSG_ANS, " ");
    }
    return super.editItems(dii, mp);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _ep.find((VEnqueteSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#selectKeys(java.util.List)
   */
  @Override
  protected List<List<String>> selectKeys(List<List<String>> keys) {
    try {
      String sfp = _keys.getRetrieve("sans");
      if ("1".equals(sfp)) {
        List<List<String>> nkey = new ArrayList<List<String>>();
        for (List<String> keylst : keys) {
          if (!_ap.isAnswer(keylst.get(0), getUserID(), sysId)) {
            nkey.add(keylst);
          }
        }
        return nkey;
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    return super.selectKeys(keys);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new EnqueteDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _ep.getDetail((EnqueteDto)dii);
  }

  private final String MSG_DEL = "*** 폜Ă܂ ***";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    if (delCheck(dii)) {
      String[] ddt = {"contents", MSG_DEL};
      return ddt;
    }
    return null;
  }

}
