package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.Calendar;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access.HATSUBAN2_KBN;
import com.hamaf.hikyakucms.sample.sns.dto.SoliloquyDto;
import com.hamaf.hikyakucms.sample.sns.dto.SoliloquySearchDto;
import com.hamaf.hikyakucms.sample.sns.process.FootprintsProcess;
import com.hamaf.hikyakucms.sample.sns.process.SoliloquyProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.SoliloquyMap;

/**
 * <p>^CgFSoliloquyList is .</p>
 * <p>FƂ茾ꗗ̕\</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/30</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class SoliloquyList extends StandardListProcess {

  private SoliloquyProcess _sp = null;
  private FootprintsProcess _fp = null;

  /**
   * Creates a new <code>SoliloquyList.java</code> instance.
   * <br>Default Constructor
   */
  public SoliloquyList() {
    _jim = new SoliloquyMap();
    _sp = new SoliloquyProcess();
    _fp = new FootprintsProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
      throws IllegalRetrieveDataException {
    SoliloquySearchDto ssd = new SoliloquySearchDto();
    String pub = keyl.getRetrieve("spublic");
    if ("1".equals(pub)) {
      ssd.setRetrieveValue(SoliloquySearchDto.TARGET, getUserID());
    } else {
      StringBuilder sb = new StringBuilder();
      sb.append("'");
      sb.append(SoliloquyProcess.PUBLIC);
      sb.append("','");
      sb.append(getUserID());
      sb.append("'");
      ssd.setRetrieveValue(SoliloquySearchDto.TARGET, sb.toString());
      ssd.setCondition(SoliloquySearchDto.TARGET, "IN");
    }

    String entdate = keyl.getRetrieve("sentday");
    if (entdate == null || entdate.length() == 0) entdate = DateUtil.getDate();
    StringBuilder sb2 = new StringBuilder();
    sb2.append("'");
    sb2.append(DateUtil.addDateTime(entdate, Calendar.MONTH, -3));
    sb2.append(" 00:00:00' AND '");
    sb2.append(entdate);
    sb2.append(" 99:99:99'");
    ssd.setRetrieveValue(SoliloquySearchDto.ENTRY_DATE, sb2.toString());
    ssd.setCondition(SoliloquySearchDto.ENTRY_DATE, "BETWEEN");

    String val = keyl.getRetrieve("sval");
    if (val.length() == 0) val = keyl.getRetrieve("key");
    if (val.length() > 0) {
      StringBuilder sb = new StringBuilder();
      sb.append("%");
      sb.append(val);
      sb.append("%");
      ssd.setRetrieveValue(SoliloquySearchDto.RET_DETAIL, sb.toString());
      ssd.setCondition(SoliloquySearchDto.RET_DETAIL, "LIKE");
    }
    ssd.setOrderItem(SoliloquySearchDto.ENTRY_DATE, -1);
    ssd.setOrderItem(SoliloquySearchDto.SEQNO, -2);
    return ssd;
  }

  private final String MSG_DSP = "dspbtn";
  private final String MSG_PRM = "parm";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii,
      Map<String, String> mp) throws Exception {
    String eu = dii.getItemValue(SoliloquyDto.ENTRY_USER);
    if (eu.equals(getUserID())) {
      mp.put(MSG_DSP, "폜");
      mp.put(MSG_PRM, "d");
    } else {
      String chk = _fp.isFootprints(HATSUBAN2_KBN.SOLILOQUY, dii.getItemValue(SoliloquyDto.SEQNO), sysId, getUserID());
      if ("1".equals(chk)) {
        mp.put(MSG_DSP, "");
        mp.put(MSG_PRM, "");
      }else {
        mp.put(MSG_DSP, "");
        mp.put(MSG_PRM, "a");
      }
    }
    return mp;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _sp.find((SoliloquySearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new SoliloquyDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _sp.getDetail((SoliloquyDto)dii);
  }

  private final String MSG_DEL = "*** 폜܂ ***";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    if (delCheck(dii)) {
      String[] ddt = {"entryName", MSG_DEL};
      return ddt;
    }
    return null;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.ListProcessBase#spareAction(com.hamaf.hikyaku.dataobject.CommonMessage, java.lang.String)
   */
  @Override
  protected CommonMessage spareAction(CommonMessage msg, String act)
      throws Exception {
    String edt = msg.getReqStr("sentday");
    if (edt == null || edt.length() == 0) {
      String[] dt = {DateUtil.getDate()};
      msg.setReq("sentday", dt);
    }
    return msg;
  }

}
