package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.Map;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationDto;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.ConversationProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.ConversionMap;

/**
 * <p>^CgFSubConvList is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/23</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class SubConvList extends StandardListProcess {

  private ConversationProcess _cp = null;
  private String _svTitle = "";
  private String _svFlg = "";
  private String _svFdate = "";

  /**
   * Creates a new <code>SubConvList.java</code> instance.
   * <br>Default Constructor
   */
  public SubConvList() {
    _jim = new ConversionMap();
    _cp = new ConversationProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
      throws IllegalRetrieveDataException {
    ConversationSearchDto csd = new ConversationSearchDto();
    csd.setRetrieveValue(ConversationSearchDto.ORIGINAL_SEQ, keyl.getRetrieve("seqNo"));
    String ttl = keyl.getRetrieve("sdetail");
    if (ttl.length() > 0) {
      csd.setRetrieveValue(ConversationSearchDto.DETAIL, "%" + ttl + "%");
      csd.setCondition(ConversationSearchDto.DETAIL, "LIKE");
    }
    csd.setOrderItem(ConversationSearchDto.SEQUENCE, 1);
    return csd;
  }

  private final String MSG_MGN = "leftmargin";
  private final String MSG_MOD = "dispmod";
  private final String MSG_REP = "disprep";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii,
      Map<String, String> mp) throws Exception {
    String ttl = dii.getItemValue(ConversationDto.TITLE);
    if (ttl.length() > 0) _svTitle = ttl;
    int ilv = Integer.parseInt(dii.getItemValue(ConversationDto.LEVEL));
    ilv --;
    mp.put(MSG_MGN, String.valueOf(ilv * 15));
    String fdt = dii.getItemValue(ConversationDto.FINISH_DATE);
    if (ilv == 0) {
      if (fdt == null || fdt.length() == 0) {
        if (getUserID().equals(dii.getItemValue(ConversationDto.ENTRY_USER))) _svFlg = "1";
      } else {
        _svFdate = fdt;
      }
    }
    if (fdt == null || fdt.length() == 0) {
      mp.put(MSG_REP, "ԐM");
      if (getUserID().equals(dii.getItemValue(ConversationDto.ENTRY_USER))) {
        mp.put(MSG_MOD, "XV");
      } else {
        mp.put(MSG_MOD, "");
      }
    } else {
      mp.put(MSG_REP, "");
      mp.put(MSG_MOD, "");
    }
    return super.editItems(dii, mp);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _cp.find((ConversationSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new ConversationDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _cp.findByPrimaryKey((ConversationDto)dii);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    return null;
  }

  private final String MSG_FDT = "finishDate";
  private final String MSG_FLG = "dspfinish";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.ProcessBase#finish(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  public CommonMessage finish(CommonMessage msg) {
    msg.setResMsg("title", _svTitle);
    msg.setResMsg(MSG_FLG, _svFlg);
    msg.setResMsg(MSG_FDT, _svFdate);
    return super.finish(msg);
  }

}
