package com.hamaf.hikyakucms.sample.wiki.ui;

import java.util.Calendar;
import java.util.Map;

import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDocDto;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDocSearchDto;
import com.hamaf.hikyakucms.sample.wiki.process.WikiDocProcess;
import com.hamaf.hikyakucms.sample.wiki.ui.mapping.WikiDocMap;

/**
 * <p>^CgFWikiList is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/09/13</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class WikiList extends StandardListProcess {

  private WikiDocProcess _wdp = null;
  private SecurityCheckUtils _scu = null;

  /**
   * Creates a new <code>WikiList</code> instance.
   * <br>Default Constructor
   */
  public WikiList() {
    _wdp = new WikiDocProcess();
    _jim = new WikiDocMap();
    _scu = new SecurityCheckUtils();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _wdp.getDocDetail((WikiDocDto)dii);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface arg0) {
    return null;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _wdp.findDoc((WikiDocSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new WikiDocDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList klst)
      throws IllegalRetrieveDataException {
    WikiDocSearchDto wdsd = new WikiDocSearchDto();
    String val = klst.getRetrieve("scategory");
    if (val != null && val.length() > 0) {
      wdsd.setRetrieveValue(WikiDocSearchDto.CATEGORY, val);
      wdsd.setCondition(WikiDocSearchDto.CATEGORY, "like");
    }
    val = klst.getRetrieve("sretval");
    if (val != null && val.length() > 0) {
      wdsd.setRetrieveValue(WikiDocSearchDto.RETVAL, "%" + val + "%");
      wdsd.setCondition(WikiDocSearchDto.RETVAL, "like");
    }
    val = klst.getRetrieve("key");
    if (val != null && val.length() > 0) {
      wdsd.setRetrieveValue(WikiDocSearchDto.RETVAL, "%" + val + "%");
      wdsd.setCondition(WikiDocSearchDto.RETVAL, "like");
    }
    val = klst.getRetrieve("allkbn");
    if (val.length() == 0) {
      String sdt = DateUtil.getDate();
      sdt = DateUtil.addDateTime(sdt, Calendar.DATE, -14);
      wdsd.setRetrieveValue(WikiDocSearchDto.LAST_DATE, sdt);
      wdsd.setCondition(WikiDocSearchDto.LAST_DATE, ">");
    }
    wdsd.setOrderItem(WikiDocSearchDto.CREATE_DATE, -1);
    wdsd.setOrderItem(WikiDocSearchDto.CREATE_TIME, -2);
    wdsd.setOrderItem(WikiDocSearchDto.TITLE, 3);
    wdsd.setOrderItem(WikiDocSearchDto.WIKI_NO, 4);
    return wdsd;
  }

  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii,
      Map<String, String> mp) throws Exception {
    String uid = dii.getItemValue(WikiDocDto.AUTHOR);
    UserDto ud =_scu.getUserInf(uid, super.getUserID());
    String mnm = ud.getItemValue(UserDto.ABBREV);
    if (mnm == null || mnm.length() == 0) mnm = ud.getItemValue(UserDto.NAME);
    mp.put(WikiDetail.CREATE_NAME, mnm);
    return mp;
  }

}
