package com.hamaf.hikyakucms.sample.wiki.util;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>^CgFWikiArticle is .</p>
 * <p>FWiki̋LǗNX</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/09/05</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class WikiArticle {
  private String _subTitle = null;
  private String _original = null;
  private String _html = null;
  private int _chapter = 0;
  private int _section = 0;
  private WikiArticle _parents = null;
  private List<WikiArticle> _children = new ArrayList<WikiArticle>();

  /**
   * Creates a new <code>WikiArticle</code> instance.
   *
   */
  public WikiArticle(String original) {
    setDocument(original);
  }
  public WikiArticle(String subtitle, String original, String html, int chapter, int section) {
    _subTitle = subtitle;
    _original = original;
    _html = html;
    _chapter = chapter;
    _section = section;
  }

  private void setDocument(String original) {
    _original = original;
    createSubTitle(original);
    StringReader sr = new StringReader(_original);
    WikiFaining wf = new WikiFaining();
    try {
      _html = wf.faining(sr);
    } catch (Exception xe) {
      xe.printStackTrace();
      _html = original;
    }
  }
  private void createSubTitle(String original) {
    if (original.length() == 0 || original.charAt(0) != '*') {
      _subTitle = "";
      return;
    }
    int stp = 1;
    if (original.length() > 1 && original.startsWith("**")) stp = 2;
    int etp = original.indexOf("\n");
    if (etp < 2) {
      _subTitle = original.substring(stp);
    } else {
      _subTitle = original.substring(stp, etp);
    }
  }

  public final String getOriginal() {
    return this._original;
  }
  public final String getHtml() {
    return this._html;
  }

  public int getChapter() {
    return _chapter;
  }
  public final void setChapter(int argChapter) {
    this._chapter = argChapter;
  }

  public final int getSection() {
    return this._section;
  }
  public final void setSection(int argSection) {
    this._section = argSection;
  }

  public final WikiArticle getParents() {
    return this._parents;
  }
  public final void setParents(final WikiArticle argParents) {
    this._parents = argParents;
  }

  public final String getSubTitle() {
    return this._subTitle;
  }
  public final void setSubTitle(final String argSubTitle) {
    this._subTitle = argSubTitle;
  }
  public final List<String> getSubTitleAll() {
    List<String> ret = new ArrayList<String>();
    if (_subTitle != null && _subTitle.length() > 0) {
      String sv = createAnchor();
      StringBuilder sb = new StringBuilder();
      sb.append(sv);
      sb.append("\t");
      sb.append(sv.substring(2));
      sb.append(". ");
      sb.append(_subTitle);
      ret.add(sb.toString());
    }
    for (WikiArticle wa : _children) {
      ret.addAll(wa.getSubTitleAll());
    }
    return ret;
  }
  private String createAnchor() {
    StringBuilder sb = new StringBuilder();
    if (_section == 0) {
      sb.append("1\t");
      sb.append(_chapter);
    } else {
      sb.append("2\t");
      sb.append(_chapter);
      sb.append("-");
      sb.append(_section);
    }
    return sb.toString();
  }

  public final List<WikiArticle> getChildren() {
    return this._children;
  }
  public final WikiArticle getChild(int pos) {
    if (pos < 1 || pos > _children.size()) return null;
    return _children.get(pos - 1);
  }
  public final void addChild(WikiArticle child) {
    child.setParents(this);
    if (_chapter == 0) {
      child.setChapter(getNextChildNo());
    } else {
      child.setChapter(_chapter);
      child.setSection(getNextChildNo());
    }
    _children.add(child);
  }
  public final void addChildOnly(WikiArticle child) {
    _children.add(child);
  }
  public final int getNextChildNo() {
    return _children.size() + 1;
  }

  public final String getOriginalAll() {
    return getDocumentAll(false, false);
  }
  public final String getHtmlAll(boolean preview) {
    return getDocumentAll(true, preview);
  }
  private final String EDIT_B =
    "\n<div class='partinfo'><a class='jump_top' href='#top' title='goto top'>[Top]</a> ";
  private final String EDIT_E =
    "<a class='partedit' title='part edit' href='dumy' onclick='edit(\"";
  private final String EDIT_A = "\"); return false;\'>[ҏW]</a>";
  private String getDocumentAll(boolean html, boolean preview) {
    StringBuilder sb = new StringBuilder();
    if (html) {
      String no = createAnchor();
      sb.append("<a name='");
      sb.append(no.substring(2));
      sb.append("'>\n");

      StringBuilder sb2 = new StringBuilder();
      sb2.append(EDIT_B);
      if (!preview) {
        sb2.append(EDIT_E);
        sb2.append(_chapter);
        sb2.append("\",\"");
        sb2.append(_section);
        sb2.append(EDIT_A);
      }
      sb2.append("</div>");

      sb.append(_html.replaceFirst("<edit>", sb2.toString()));
    } else {
      sb.append(_original);
    }
    for (WikiArticle wa : _children) {
      if (html) {
        sb.append(wa.getHtmlAll(preview));
      } else {
        sb.append(wa.getOriginalAll());
      }
    }
    return sb.toString();
  }

}
