package com.hamaf.hikyakucms.sample.wiki.util;

import java.io.BufferedReader;
import java.io.StringReader;
import java.util.List;

/**
 * <p>
 * ^CgFWikiDocument is .
 * </p>
 * <p>
 * FWikiǗ
 * </p>
 * <br>
 *
 * <p>
 * 쌠: Copyright (c) 2012/09/05
 * </p>
 *
 * @author Yoshinori
 * @version 1.0
 *
 */
public class WikiDocument {

  private String _title = null;
  private String _category = null;
  private String _summary = null;
  private WikiArticle _doc = null;
  private String _author = null;
  private String _creatinDate = null;
  private String _creatinTime = null;
  private String _lastAuthor = null;
  private String _lastDate = null;
  private String _lastTime = null;

  /**
   * Creates a new <code>WikiDocument</code> instance.
   *
   */
  public WikiDocument() {
  }

  public final WikiArticle getDoc() {
    return this._doc;
  }

  public final void setDoc(final WikiArticle argDoc) {
    this._doc = argDoc;
  }

  public final void setHeader(String title, String category, String summary) {
    this._title = title;
    this._category = category;
    this._summary = summary;
  }

  public final void setCreate(String author, String date, String time) {
    this._author = author;
    this._creatinDate = date;
    this._creatinTime = time;
  }

  public final void setLast(String author, String date, String time) {
    this._lastAuthor = author;
    this._lastDate = date;
    this._lastTime = time;
  }

  public void setDocument(String data, String chapter, String section) throws Exception {
    int lvl = getLevel(chapter, section);
    StringReader sr = new StringReader(data);
    BufferedReader brd = new BufferedReader(sr);
    String buf = brd.readLine();
    StringBuilder sb = new StringBuilder();
    boolean isVerbatim = false;
    while (buf != null) {
      if (buf.startsWith("--(") || buf.startsWith("---(")) {
        isVerbatim = true;
      }
      if (buf.startsWith("--)") || buf.startsWith("---)")) {
        isVerbatim = false;
      }
      if (!isVerbatim && buf.length() > 0 && buf.charAt(0) == '*') {
        if (lvl > 1) {
          if (buf.length() < lvl || buf.charAt(lvl - 1) != '*') buf = " "  + buf;
        }
        if (!(buf.length() > 3 && buf.startsWith("***"))) {
          if ("0".equals(chapter) || sb.length() > 0) {
            creDoc(sb.toString(), chapter, section);
          }
          sb = new StringBuilder();
        }
      }
      sb.append(buf);
      sb.append("\n");
      buf = brd.readLine();
    }
    creDoc(sb.toString(), chapter, section);
    sr.close();
  }

  private int getLevel(String chapter, String section) {
    int lvl = 0;
    if (!"0".equals(chapter)) {
      lvl++;
      if (!"0".equals(section)) lvl++;
    }
    return lvl;
  }

  private WikiArticle oya = null;

  private void creDoc(String data, String chapter, String section) {
    WikiArticle wa = new WikiArticle(data);
    if (_doc == null) {
      wa.setChapter(Integer.parseInt(chapter));
      wa.setSection(Integer.parseInt(section));
      _doc = wa;
    } else {
      if (data.length() > 2 && data.startsWith("**")) {
        if (oya == null) {
          _doc.addChild(wa);
        } else {
          oya.addChild(wa);
        }
      } else {
        _doc.addChild(wa);
        oya = wa;
      }
    }
  }

  public String getOriginal() {
    return _doc.getOriginalAll();
  }

  public String getDiffDoc() {
    StringBuilder sb = new StringBuilder();
    sb.append("^Cg:");
    sb.append(_title);
    sb.append("\nJeS[:");
    sb.append(_category);
    sb.append("\nT:");
    sb.append(_summary);
    sb.append("\n");
    sb.append(getOriginal());
    return sb.toString();
  }

  private final String BACK =
      "<a class='partedit' title='back' href='dumy' onclick='back(); return false;\'>[߂]</a>";
  private final String EDIT_E =
    "<a class='partedit' title='edit' href='dumy' onclick='editDoc(); return false;\'>[ҏW]</a>";
  private final String EDIT_D =
    "<a class='partedit' title='diff' href='dumy' onclick='diff(); return false;\'>[ύX]</a>";
  public String getHtml(boolean preview, boolean diff) {
    StringBuilder sb = new StringBuilder();
    sb.append("<a name='top'>\n");
    if (!preview) {
      sb.append(BACK);
      sb.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
      sb.append(EDIT_E);
      sb.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
      if (diff) sb.append(EDIT_D);
    }
    sb.append("\n");
    sb.append("<h1>");
    sb.append(_title);
    sb.append(" y");
    sb.append(_category);
    sb.append("z</h1>\n");
    sb.append("<h5>");
    sb.append(_summary);
    sb.append("</h5>\n");
    sb.append("<address class='lastupdate'>o^ҁF");
    sb.append(_author);
    sb.append(" iF");
    sb.append(_creatinDate);
    sb.append(" ");
    sb.append(_creatinTime);
    sb.append("j &nbsp;&nbsp;&nbsp; ŏIXVF");
    sb.append(_lastAuthor);
    sb.append(" iF");
    sb.append(_lastDate);
    sb.append(" ");
    sb.append(_lastTime);
    sb.append("j</address>\n");

    sb.append("<h2 class='toc'>ڎ</h2>\n");
    int svn = 0;
    List<String> idxs = getIndexInf();
    for (String idx : idxs) {
      String[] idxdt = idx.split("\t");
      int no = Integer.parseInt(idxdt[0]);
      if (no != svn) {
        if (no > svn) {
          sb.append("<ul>\n");
        } else {
          sb.append("</ul>\n");
        }
        svn = no;
      }
      sb.append("<li><a href='#");
      sb.append(idxdt[1]);
      sb.append("'>");
      sb.append(idxdt[2]);
      sb.append("</a>\n");
    }
    sb.append("</ul>\n<p/>\n");

    sb.append(_doc.getHtmlAll(preview));
    return sb.toString();
  }

  public List<String> getIndexInf() {
    return _doc.getSubTitleAll();
  }

  public void setSubject(String summary) {
    this._summary = summary;
  }

  public String getSubject() {
    return _summary;
  }

}
