package com.hamaf.business.generaldoc.process;

import com.hamaf.business.generaldoc.access.GeneralDocAccess;
import com.hamaf.business.generaldoc.dto.GeneralDocDto;
import com.hamaf.business.generaldoc.dto.GeneralDocSearchDto;
import com.hamaf.business.workflow.WorkFlowUpdate;
import com.hamaf.business.workflow.access.HatsubanAccess;
import com.hamaf.business.workflow.access.HatsubanAccess.HATSUBAN_KBN;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.USE_FLG;
import com.hamaf.hikyaku.util.DateUtil;

/**
 * <p>^CgFGeneralDocProcess is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/10/27</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class GeneralDocProcess implements WorkFlowUpdate {

  private GeneralDocAccess _gda = null;
  private HatsubanAccess _ha = null;
  
  /**
   * Creates a new <code>GeneralDocProcess</code> instance.
   * <br>Default Constructor
   */
  public GeneralDocProcess() {
    _gda = new GeneralDocAccess();
    _ha = new HatsubanAccess();
  }
  
  public GeneralDocDto getDetaail(GeneralDocDto gdd) throws Exception {
    return (GeneralDocDto)_gda.findByPrimaryKey(gdd);
  }
  public GeneralDocSearchDto find(GeneralDocSearchDto gdsd) throws Exception {
    return (GeneralDocSearchDto)_gda.find(gdsd);
  }
  public String addNew(GeneralDocDto gdd) throws Exception {
    String no = _ha.getNextNo(HATSUBAN_KBN.GENERAL, null, gdd.getSysId());
    gdd.setItemValue(GeneralDocDto.SEQNO, no);
    gdd.setItemValue(GeneralDocDto.APPROVAL_RESULT, "S");
    gdd.setItemValue(GeneralDocDto.ADD_DATE, DateUtil.getDate());
    gdd.setItemValue(GeneralDocDto.ADD_USER, gdd.getCommFld(COMMFLD.UUSR));
    _gda.regist(gdd);
    return no;
  }
  public void update(GeneralDocDto gdd) throws Exception {
    _gda.update(gdd);
  }
  public void delete(GeneralDocDto gdd) throws Exception {
    gdd.setCommFld(COMMFLD.DEL, USE_FLG.DEL.getVal());
    update(gdd);
  }
  
  private final String SYS_ID = "workflow";
  @Override
  public void registFlowNo(String key, String no) throws Exception {
    GeneralDocDto gdd = getDto(key);
    if (gdd.isNotFound()) return;
    gdd.setItemValue(GeneralDocDto.APPROVAL_NO, no);
    if (no == null || no.length() == 0) {
      gdd.setItemValue(GeneralDocDto.APPROVAL_RESULT, "S");
    } else {
      gdd.setItemValue(GeneralDocDto.APPROVAL_RESULT, "I");
    }
    _gda.update(gdd);
  }

  @Override
  public void registResult(String key, String result) throws Exception {
    GeneralDocDto gdd = getDto(key);
    if (gdd.isNotFound()) return;
    gdd.setItemValue(GeneralDocDto.APPROVAL_RESULT, result);
    _gda.update(gdd);
  }
  private GeneralDocDto getDto(String key) throws Exception {
    GeneralDocDto gdd = new GeneralDocDto();
    gdd.setSysId(SYS_ID);
    gdd.setItemValue(GeneralDocDto.SEQNO, key);
    gdd = (GeneralDocDto)_gda.findByPrimaryKey(gdd);
    return gdd;
  }
}
