package com.hamaf.business.workflow.ui;

import java.util.ArrayList;
import java.util.List;

import com.hamaf.business.workflow.access.BaseLevelAccess.AUTHORIZER_KUBUN;
import com.hamaf.business.workflow.access.BaseLevelAccess.CHECK_TYPE;
import com.hamaf.business.workflow.dto.BaseFlowDto;
import com.hamaf.business.workflow.dto.BaseLevelDto;
import com.hamaf.business.workflow.process.BaseFlowProcess;
import com.hamaf.business.workflow.ui.mapping.BaseFlowMapping;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.ui.StandardCheckAndUpdate;

/**
 * <p>^CgFBaseFlowAdd is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/10/09</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class BaseFlowAdd extends StandardCheckAndUpdate {

  private BaseFlowProcess _bfp;
  
  /**
   * Creates a new <code>BaseFlowAdd</code> instance.
   * <br>Default Constructor
   */
  public BaseFlowAdd() {
    _cfld = "BaseFlowAdd";
    _jim = new BaseFlowMapping();
    _bfp = new BaseFlowProcess();
  }

  private final String UPKBN = "upkbn";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface, com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage callControl(DetailItemsInterface dii,
      CommonMessage msg) throws Exception {
    List<BaseLevelDto> blds = getLevelInf(msg);

    String kbn = msg.getReqStr(UPKBN).toLowerCase();
    if (kbn.startsWith("a")) {
      _bfp.addNewFlow((BaseFlowDto)dii, blds);
    } else if (kbn.startsWith("m")) {
      _bfp.updateFlow((BaseFlowDto)dii, blds);
    }
    
    return msg;
  }

  public static final String AUTH_USER = "auser";
  public static final String AUTH_GROUP = "agroup";
  public static final String NOTI_USER = "nuser";
  public static final String NOTI_GROUP = "ngroup";
  public static final String CHK_TYPE = "chktyp";
  private List<BaseLevelDto> getLevelInf(CommonMessage msg) {
    List<BaseLevelDto> blds = new ArrayList<BaseLevelDto>();
    int lv  = 0;
    String svnus = null;
    String svngp = null;
    for (int i = 1; i < 10; i++) {
      String aus = msg.getReqStr(AUTH_USER + i);
      String agp = msg.getReqStr(AUTH_GROUP + i);
      String nus = msg.getReqStr(NOTI_USER + i);
      String ngp = msg.getReqStr(NOTI_GROUP + i);
      if (aus.length() + agp.length() > 0) {
        lv++;
        BaseLevelDto bld = createLevelDto(msg.getReqStr("flowNo"), lv,
            AUTHORIZER_KUBUN.AUTHORIZE, msg.getReqStr(CHK_TYPE + i), aus, agp);
        blds.add(bld);
        if (nus.length() + ngp.length() > 0) {
          BaseLevelDto nbld = createLevelDto(msg.getReqStr("flowNo"), lv,
              AUTHORIZER_KUBUN.NOTIFICATION, CHECK_TYPE.ALL.getVal(), nus, ngp);
          blds.add(nbld);
        }
        svnus = null;
        svngp = null;
      } else {
        if (nus.length() + ngp.length() > 0) {
          svnus = nus;
          svngp = ngp;
        }
      }
    }
    if (svnus != null) {
      lv++;
      BaseLevelDto nbld = createLevelDto(msg.getReqStr("flowNo"), lv,
          AUTHORIZER_KUBUN.NOTIFICATION, CHECK_TYPE.ALL.getVal(), svnus, svngp);
      blds.add(nbld);
    }
    return blds;
  }
  private BaseLevelDto createLevelDto(String flowNo, int level, AUTHORIZER_KUBUN kbn,
      String check, String user, String group) {
    BaseLevelDto bld = new BaseLevelDto();
    bld.setSysId(sysId);
    bld.setCommFld(COMMFLD.UUSR, getUserID());
    bld.setCommFld(COMMFLD.UPGM, this.getClass().getName());
    bld.setItemValue(BaseLevelDto.FLOW_NO, flowNo);
    bld.setItemValue(BaseLevelDto.LEVEL, String.valueOf(level));
    bld.setItemValue(BaseLevelDto.AUTHORIZER_KUBUN, kbn.getVal());
    bld.setItemValue(BaseLevelDto.CHECK_TYPE, check);
    bld.setItemValue(BaseLevelDto.FLOW_NO, flowNo);
    bld.setGroups(group);
    bld.setUsers(user);
    return bld;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardCheckAndUpdate#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new BaseFlowDto();
  }

}
