/**
 * 
 */
package com.hamaf.business.workflow.ui;

import java.util.List;
import java.util.Map;

import com.hamaf.business.workflow.access.BaseLevelAccess.AUTHORIZER_KUBUN;
import com.hamaf.business.workflow.dto.BaseFlowDto;
import com.hamaf.business.workflow.dto.BaseLevelDto;
import com.hamaf.business.workflow.dto.BaseLevelSearchDto;
import com.hamaf.business.workflow.process.BaseFlowProcess;
import com.hamaf.business.workflow.ui.mapping.BaseLevelMapping;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;

/**
 * <p>^CgFBaseLevelList is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/10/10</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class BaseLevelList extends StandardListProcess {

  private BaseFlowProcess _bfp = null;
  
  /**
   * Creates a new <code>BaseLevelList</code> instance.
   * <br>Default Constructor
   */
  public BaseLevelList() {
    _jim = new BaseLevelMapping();
    _bfp = new BaseFlowProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _bfp.getLeveDetail((BaseLevelDto)dii);
  }

  private final String MSG_USERS = "user";
  private final String MSG_GROUPS = "group";
  private final String MSG_UNAMES = "uname";
  private final String MSG_GNAMES = "gname";
  @Override
  protected Map<String, String> editItems(DetailItemsInterface argDii,
      Map<String, String> argMp) throws Exception {
    BaseLevelDto bld = (BaseLevelDto)argDii;
    argMp.put(MSG_USERS, bld.getUsers());
    argMp.put(MSG_UNAMES, bld.getUnames());
    argMp.put(MSG_GROUPS, bld.getGroups());
    argMp.put(MSG_GNAMES, bld.getGnames());
    return argMp;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface argArg0) {
    return null;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _bfp.findLevel((BaseLevelSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new BaseLevelDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList klst)
      throws IllegalRetrieveDataException {
    BaseLevelSearchDto blsd = new BaseLevelSearchDto();
    blsd.setRetrieveValue(BaseLevelSearchDto.FLOW_NO, klst.getRetrieve("flowNo"));
    return blsd;
  }

  @Override
  public CommonMessage finish(CommonMessage argMsg) {
    BaseFlowDto bfd = new BaseFlowDto();
    bfd.setSysId(sysId);
    bfd.setItemValue(BaseFlowDto.FLOW_NO, argMsg.getReqStr("flowNo"));
    bfd.setAllRequest();
    try {
      bfd = (BaseFlowDto)_bfp.getFlowDetail(bfd);
      argMsg.setResMsg("flowNo", bfd.getItemValue(BaseFlowDto.FLOW_NO));
      argMsg.setResMsg("name", bfd.getItemValue(BaseFlowDto.NAME));
      argMsg.setResMsg("summary", bfd.getItemValue(BaseFlowDto.SUMMARY));
      argMsg.setResMsg("lvlCnt", bfd.getItemValue(BaseFlowDto.LEVEL_COUNT));
      for (COMMFLD fld : COMMFLD.values()) {
        argMsg.setResMsg(fld.getVal(), bfd.getCommFld(fld));
      }
    } catch (Exception xe) {
      xe.printStackTrace();
    }
    argMsg = cnvLvlDt(argMsg);
    return argMsg;
  }
  private CommonMessage cnvLvlDt(CommonMessage msg) {
    List<Map<String, String>> ldt = msg.getResList();
    for (Map<String, String> dt : ldt) {
      String uid = null;
      String gid = null;
      String lvl = dt.get("level");
      if (dt.get("authoKbn").equals(AUTHORIZER_KUBUN.AUTHORIZE.getVal())) {
        uid = BaseFlowAdd.AUTH_USER;
        gid = BaseFlowAdd.AUTH_GROUP;
      } else {
        uid = BaseFlowAdd.NOTI_USER;
        gid = BaseFlowAdd.NOTI_GROUP;
      }
      String unm = uid + "Nm" + lvl;
      String unms = unm + "s";
      String gnm = gid + "Nm" + lvl;
      String gnms = gnm + "s";

      msg.setResMsg(uid + lvl, dt.get(MSG_USERS));
      msg.setResMsg(gid + lvl, dt.get(MSG_GROUPS));
      msg.setResMsg(unm, dt.get(MSG_UNAMES));
      msg.setResMsg(gnm, dt.get(MSG_GNAMES));
      msg.setResMsg(unms, dt.get(MSG_UNAMES).replaceAll(",", "\n"));
      msg.setResMsg(gnms, dt.get(MSG_GNAMES).replaceAll(",", "\n"));
    }
    return msg;
  }

}
